PC_header = {
    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36"
}

jianshu_request_header = {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36 Edg/89.0.774.57",
                          "X-INFINITESCROLL": "true",
                          "X-Requested-With": "XMLHttpRequest"
                          }

BeikeIsland_request_header = {"Host": "www.beikeisland.com",
                            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36 Edg/89.0.774.57",
                            "Content-Type": "application/json",
                            "Version": "v2.0"
                            }


from datetime import datetime

from .assert_funcs import AssertCollectionUrl
from .basic_apis import (GetCollectionArticlesJsonDataApi,
                         GetCollectionEditorsJsonDataApi,
                         GetCollectionJsonDataApi,
                         GetCollectionRecommendedWritersJsonDataApi,
                         GetCollectionSubscribersJsonDataApi)
from .convert import CollectionUrlToCollectionSlug


def GetCollectionName(collection_url: str) -> str:
    """该函数接收专题 Url，并返回该链接对应专题的名称

    Args:
        collection_url (str): 专题 Url

    Returns:
        str: 链接对应专题的名称
    """
    AssertCollectionUrl(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = json_obj["title"]
    return result

def GetCollectionAvatarUrl(collection_url: str) -> str:
    """获取专题头像链接

    Args:
        collection_url (str): 专题 Url

    Returns:
        str: 专题头像链接
    """
    AssertCollectionUrl(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = json_obj["image"]
    return result

def GetCollectionIntroductionText(collection_url: str) -> str:
    """该函数接收专题 Url，并返回该链接对应专题的简介

    Args:
        collection_url (str): 专题 Url

    Returns:
        str: 链接对应专题的简介
    """
    AssertCollectionUrl(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = json_obj["content_without_html"]
    return result

def GetCollectionIntroductionHtml(collection_url: str) -> str:
    """该函数接收专题 Url，并返回该链接对应专题的简介

    Args:
        collection_url (str): 专题 Url

    Returns:
        str: 链接对应专题的简介
    """
    AssertCollectionUrl(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = json_obj["content_in_full"]
    return result

def GetCollectionArticlesCount(collection_url: str) -> int:
    """该函数接收专题 Url，并返回该链接对应专题的收录文章数量

    Args:
        collection_url (str): 专题 Url

    Returns:
        int: 链接对应专题的收录文章数量
    """
    AssertCollectionUrl(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = json_obj["notes_count"]
    return result

def GetCollectionSubscribersCount(collection_url: str) -> int:
    """该函数接收专题 Url，并返回该链接对应专题的关注者数量

    Args:
        collection_url (str): 专题 Url

    Returns:
        int: 链接对应专题的关注者数量
    """
    AssertCollectionUrl(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = json_obj["subscribers_count"]
    return result

def GetCollectionArticlesUpdateTime(collection_url: str) -> datetime:
    """获取专题文章更新时间

    Args:
        collection_url (str): 专题 Url

    Returns:
        datetime: 专题文章更新时间
    """
    AssertCollectionUrl(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = datetime.fromtimestamp(json_obj["newly_added_at"])
    return result

def GetCollectionInfoUpdateTime(collection_url: str) -> datetime:
    """获取专题信息更新时间

    Args:
        collection_url (str): 专题 Url

    Returns:
        datetime: 专题信息更新时间
    """
    AssertCollectionUrl(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = datetime.fromtimestamp(json_obj["last_updated_at"])
    return result

def GetCollectionOwnerInfo(collection_url: str) -> dict:
    """获取专题的所有者信息

    Args:
        collection_url (str): 专题 Url

    Returns:
        dict: 用户信息
    """
    AssertCollectionUrl(collection_url)
    json_obj = GetCollectionJsonDataApi(collection_url)
    result = {
        "uid": json_obj["owner"]["id"], 
        "name": json_obj["owner"]["nickname"], 
        "uslug": json_obj["owner"]["slug"]
    }
    return result

def GetCollectionEditorsInfo(collection_id: int, page: int = 1) -> list:
    """该函数接收专题 ID，并返回 ID 对应专题的编辑信息

    Args:
        collection_id (int): 专题 ID
        page (int, optional): 页码. Defaults to 1.

    Returns:
        list: ID 对应专题的编辑信息
    """
    json_obj = GetCollectionEditorsJsonDataApi(collection_id, page=page)
    result = []
    for item in json_obj["editors"]:
        item_data = {
            "uslug": item["slug"], 
            "name": item["nickname"], 
            "avatar_url": item["avatar_source"]
        }
        result.append(item_data)
    return result

def GetCollectionRecommendedWritersInfo(collection_id: int, page: int = 1, count: int = 20) -> list:
    """该函数接收一个专题 ID，并返回该 ID 对应专题的推荐作者信息

    Args:
        collection_id (int): 专题 ID
        page (int, optional): 页码. Defaults to 1.
        count (int, optional): 每次返回的数量. Defaults to 20.

    Returns:
        list: 推荐作者信息
    """
    json_obj = GetCollectionRecommendedWritersJsonDataApi(collection_id, page=page, count=count)
    result = []
    for item in json_obj["users"]:
        item_data = {
            "uid": item["id"], 
            "uslug": item["slug"], 
            "name": item["nickname"], 
            "avatar_url": item["avatar_source"], 
            "collection_name": item["collection_name"], 
            "likes_count": item["total_likes_count"], 
            "words_count": item["total_wordage"]
        }
        result.append(item_data)
    return result

def GetCollectionSubscribersInfo(collection_id: int, start_sort_id: int = None) -> list:
    """该函数接收一个专题 ID，并返回该 ID 对应专题的关注者信息

    Args:
        collection_id (int): 专题 ID
        start_sort_id (int): 起始序号，等于上一条数据的序号

    Returns:
        list: 关注者信息
    """
    json_obj = GetCollectionSubscribersJsonDataApi(collection_id, max_sort_id=start_sort_id)
    result = []
    for item in json_obj:
        item_data = {
            "uslug": item["slug"], 
            "name": item["nickname"], 
            "avatar_url": item["avatar_source"], 
            "sort_id": item["like_id"], 
            "subscribe_time": datetime.fromisoformat(item["subscribed_at"])
        }
        result.append(item_data)
    return result

def GetCollectionArticlesInfo(collection_url: str, page: int = 1, 
                                count: int = 10, sorting_method: str ="time") -> list:
    """该函数接收专题 Url ，并返回该 Url 对应专题的文章信息

    Args:
        collection_url (str): 专题 Url
        page (int, optional): 页码. Defaults to 1.
        count (int, optional): 每次返回的数据数量. Defaults to 10.
        sorting_method (str, optional): 排序方法，time 为按照发布时间排序，
        comment_time 为按照最近评论时间排序，hot 为按照热度排序. Defaults to "time".

    Returns:
        list: 文章信息
    """
    AssertCollectionUrl(collection_url)
    order_by = {
        "time": "added_at", 
        "comment_time": "commented_at", 
        "hot": "top"
    }[sorting_method]
    json_obj = GetCollectionArticlesJsonDataApi(CollectionUrlToCollectionSlug(collection_url), 
                                             page=page, count=count, order_by=order_by)
    result = []
    for item in json_obj:
        item_data  = {
            "aid": item["object"]["data"]["id"], 
            "title": item["object"]["data"]["title"], 
            "aslug": item["object"]["data"]["slug"], 
            "release_time": datetime.fromisoformat(item["object"]["data"]["first_shared_at"]), 
            "first_image_url": item["object"]["data"]["list_image_url"], 
            "summary": item["object"]["data"]["public_abbr"], 
            "views_count": item["object"]["data"]["views_count"], 
            "likes_count": item["object"]["data"]["likes_count"], 
            "paid": item["object"]["data"]["paid"], 
            "commentable": item["object"]["data"]["commentable"], 
            "user": {
                "uid": item["object"]["data"]["user"]["id"], 
                "name": item["object"]["data"]["user"]["nickname"], 
                "uslug": item["object"]["data"]["user"]["slug"], 
                "avatar_url": item["object"]["data"]["user"]["avatar"]
            }, 
            "total_fp_amount": item["object"]["data"]["total_fp_amount"] / 1000, 
            "comments_count": item["object"]["data"]["public_comments_count"], 
            "rewards_count": item["object"]["data"]["total_rewards_count"]
        }
        result.append(item_data)
    return result



try:
    import simplejson as json
except ImportError:
    import json

from functools import lru_cache

from .basic_apis import GetArticleJsonDataApi, GetUserJsonDataApi
from .exceptions import InputError, ResourceError
from .headers import jianshu_request_header


def AssertString(object: any) -> None:
    if isinstance(object, str) == False:
        raise TypeError("传入的数据不是字符串")

def AssertInt(object: any) -> None:
    if isinstance(object, int) == False:
        raise TypeError("传入的数据不是整数")

def AssertFloat(object: any) -> None:
    if isinstance(object, float) == False:
        raise TypeError("传入的数据不是浮点数")


def AssertJianshuUrl(string: str) -> None:
    """判断是否是有效的简书 Url

    Args:
        string (str): 需要进行判断的字符串

    Raises:
        InputError: 传入的参数不是有效的简书 Url 时抛出此错误
    """
    keyword_to_find = ["https://", "www.jianshu.com"]
    for keyword in keyword_to_find:
        if string.find(keyword) == -1:
            raise InputError("参数" + string + "不是有效的简书 Url")


def AssertUserUrl(string: str) -> None:
    """判断是否是有效的简书用户主页 Url

    Args:
        string (str): 需要进行判断的字符串

    Raises:
        InputError: 传入的参数不是有效的简书用户主页 Url 时抛出此错误
    """
    keyword_to_find = ["https://", "www.jianshu.com", "/u/"]
    for keyword in keyword_to_find:
        if string.find(keyword) == -1:
            raise InputError("参数" + string + "不是有效的简书用户主页 Url")

@lru_cache
def AssertUserStatusNormal(user_url: str) -> None:
    user_json_data = GetUserJsonDataApi(user_url)
    try:
        user_json_data["nickname"]
    except KeyError:
        raise ResourceError("用户账号状态异常")

def AssertArticleUrl(string: str) -> None:
    """判断是否是有效的简书文章 Url

    Args:
        string (str): 需要进行判断的字符串

    Raises:
        InputError: 传入的参数不是有效的简书文章 Url 时抛出此错误
    """
    keyword_to_find = ["https://", "www.jianshu.com", "/p/"]
    for keyword in keyword_to_find:
        if string.find(keyword) == -1:
            raise InputError("参数" + string + "不是有效的简书文章 Url")

@lru_cache
def AssertArticleStatusNormal(article_url: str) -> None:
    """判断文章状态是否正常

    Args:
        article_url (str): 文章 Url

    Raises:
        ResourceError: 文章状态不正常时抛出此错误
    """
    AssertArticleUrl(article_url)
    json_obj = GetArticleJsonDataApi(article_url)
    try:
        json_obj["show_ad"]
    except KeyError:
        raise ResourceError("文章状态异常")

def AssertNotebookUrl(string: str) -> None:
    """判断是否是有效的简书文集 Url

    Args:
        string (str): 需要进行判断的字符串

    Raises:
        InputError: 传入的参数不是有效的简书文集 Url 时抛出此错误
    """
    keyword_to_find = ["https://", "www.jianshu.com", "/nb/"]
    for keyword in keyword_to_find:
        if string.find(keyword) == -1:
            raise InputError("参数" + string + "不是有效的简书文集 Url")
        
def AssertCollectionUrl(string: str) -> None:
    """判断是否是有效的简书专题 Url

    Args:
        string (str): 需要进行判断的字符串

    Raises:
        InputError: 传入的参数不是有效的简书专题 Url 时抛出此错误
    """
    keyword_to_find = ["https://", "www.jianshu.com", "/c/"]
    for keyword in keyword_to_find:
        if string.find(keyword) == -1:
            raise InputError("参数" + string + "不是有效的简书专题 Url")


def AssertIslandUrl(string: str) -> None:
    """判断是否是有效的简书小岛 Url

    Args:
        string (str): 需要进行判断的字符串

    Raises:
        InputError: 传入的参数不是有效的简书小岛 Url 时抛出此错误
    """
    keyword_to_find = ["https://", "www.jianshu.com", "/g/"]
    for keyword in keyword_to_find:
        if string.find(keyword) == -1:
            raise InputError("参数" + string + "不是有效的简书小岛 Url")



__version__ = "2.2.0"

from . import (article, beikeisland, collection, island, notebook, objects,
               rank, user)


def future():
    print("回望洪荒，历史苍茫；\n只身向前，重铸星光。\n赴梦万里，道阻且长；\n今日萤火，明日华章。")



from datetime import datetime
from re import findall, sub

from lxml import etree

from .assert_funcs import AssertArticleStatusNormal, AssertArticleUrl
from .basic_apis import (GetArticleCommentsJsonDataApi,
                         GetArticleHtmlJsonDataApi, GetArticleJsonDataApi)
from .headers import PC_header, jianshu_request_header


def GetArticleTitle(article_url: str) -> str:
    """获取文章标题

    Args:
        article_url (str): 文章 Url

    Returns:
        str: 文章标题
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleJsonDataApi(article_url)
    result = json_obj["public_title"]
    return result

def GetArticleAuthorName(article_url: str) -> str:
    """获取文章作者名

    Args:
        article_url (str): 文章 Url

    Returns:
        str: 文章作者名
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleHtmlJsonDataApi(article_url)
    result = json_obj["props"]["initialState"]["note"]["data"]["user"]["nickname"]
    return result

def GetArticleReadsCount(article_url: str) -> str:
    """获取文章阅读量

    Args:
        article_url (str): 文章 Url

    Returns:
        str: 文章阅读量
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleHtmlJsonDataApi(article_url)
    result = json_obj["props"]["initialState"]["note"]["data"]["views_count"]
    return result

def GetArticleWordage(article_url: str) -> str:
    """获取文章总字数

    Args:
        article_url (str): 文章 Url

    Returns:
        str: 文章总字数
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleHtmlJsonDataApi(article_url)
    result = json_obj["props"]["initialState"]["note"]["data"]["wordage"]
    return result

def GetArticleLikesCount(article_url: str) -> int:
    """获取文章点赞量

    Args:
        article_url (str): 文章 Url

    Returns:
        int: 文章点赞量
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleJsonDataApi(article_url)
    result = json_obj["likes_count"]
    return result

def GetArticleCommentsCount(article_url: str) -> int:
    """获取文章评论量

    Args:
        article_url (str): 文章 Url

    Returns:
        int: 文章评论量
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleJsonDataApi(article_url)
    result = json_obj["public_comment_count"]
    return result

def GetArticleMostValuableCommentsCount(article_url: str) -> int:
    """获取文章精选评论量

    Args:
        article_url (str): 文章 Url

    Returns:
        int: 文章精选评论量
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleJsonDataApi(article_url)
    result = json_obj["featured_comments_count"]
    return result

def GetArticleTotalFPCount(article_url: str) -> int:
    """获取文章总获钻量

    Args:
        article_url (str): 文章 Url

    Returns:
        int: 文章总获钻量
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleJsonDataApi(article_url)
    result = json_obj["total_fp_amount"] / 1000
    return result

def GetArticleDescription(article_url: str) -> str:
    """获取文章摘要

    Args:
        article_url (str): 文章 Url

    Returns:
        str: 文章摘要
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleJsonDataApi(article_url)
    result = json_obj["description"]
    return result

def GetArticlePublishTime(article_url: str) -> datetime:
    """获取文章发布时间

    Args:
        article_url (str): 文章 Url

    Returns:
        datetime: 文章发布时间
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleJsonDataApi(article_url)
    result = datetime.fromisoformat(json_obj["first_shared_at"])
    return result

def GetArticleUpdateTime(article_url: str) -> datetime:
    """获取文章更新时间

    Args:
        article_url (str): 文章 Url

    Returns:
        datetime: 文章更新时间
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleJsonDataApi(article_url)
    result = datetime.fromtimestamp(json_obj["last_updated_at"])
    return result

def GetArticlePaidStatus(article_url: str) -> bool:
    """获取文章付费状态

    Args:
        article_url (str): 文章 Url

    Returns:
        bool: 文章付费状态，True 为付费文章，False 为免费文章
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleJsonDataApi(article_url)
    paid_type = {
        "free": False, 
        "fbook_free": False, 
        "pbook_free":False, 
        "paid": True, 
        "fbook_paid": True, 
        "pbook_paid":True
    }
    result = paid_type[json_obj["paid_type"]]
    return result

def GetArticleReprintStatus(article_url: str) -> bool:
    """获取文章转载声明状态

    Args:
        article_url (str): 文章 Url

    Returns:
        bool: 文章转载声明状态，True 为允许转载，False 为禁止转载
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleJsonDataApi(article_url)
    result = json_obj["reprintable"]
    return result

def GetArticleCommentStatus(article_url: str) -> bool:
    """获取文章评论状态

    Args:
        article_url (str): 文章 Url

    Returns:
        bool: 文章评论状态，True 为开启评论，False 为关闭评论
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleJsonDataApi(article_url)
    result = json_obj["commentable"]
    return result


def GetArticleHtml(article_url: str) -> str:
    """获取 Html 格式的文章内容

    # ! 该函数可以获取设置禁止转载的文章内容，请尊重作者版权，由此带来的风险由您自行承担
    # ! 该函数不能获取需要付费的文章内容
    # ! 文章中的图片描述将会丢失

    Args:
        article_url (str): 文章 Url

    Returns:
        str: Html 格式的文章内容
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleJsonDataApi(article_url)
    html_text = json_obj["free_content"]
    html_text = sub(r'<div class="image-[\w]*" [ \w+-="]*>', "", html_text)  # 去除 image-view 和 image-container
    html_text = sub(r'<div class="image-package">', "", html_text)  # 去除 image-package
    html_text = sub(r'<div class="image-container-fill".+>', "", html_text)  # 去除 image-container-fill
    old_img_blocks = findall(r'\<img[ \w+-="]*>', html_text)  # 匹配旧的 img 标签
    img_names = findall(r"\w+-\w+.[jpg | png]{3}",html_text)  # 获取图片名称
    new_img_blocks = ["".join(['<img src="https://upload-images.jianshu.io/upload_images/', \
                    img_name, '">']) for img_name in img_names]  # 拼接新的 img 标签
    if len(old_img_blocks) == 0:
        replaced = html_text
    for index in range(len(old_img_blocks)):
        if index == 0:
            replaced = html_text.replace(old_img_blocks[index], new_img_blocks[index])
        else:
            replaced = replaced.replace(old_img_blocks[index], new_img_blocks[index])  # 替换 img 标签
    return replaced

def GetArticleText(article_url: str) -> str:
    """获取纯文本格式的文章内容

    # ! 该函数可以获取设置禁止转载的文章内容，请尊重作者版权，由此带来的风险由您自行承担
    # ! 该函数不能获取需要付费的文章内容

    Args:
        article_url (str): 文章 Url

    Returns:
        str: 纯文本格式的文章内容
    """
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleJsonDataApi(article_url)
    html_text = json_obj["free_content"]
    html_obj = etree.HTML(html_text)
    result = "".join(html_obj.itertext())
    result = sub(r"\s{2,}", "", result)
    return result

def GetArticleCommentsData(article_id: int, page: int = 1, count: int = 10, 
                           author_only: bool = False, sorting_method: str = "positive") -> list:
    """获取文章评论信息

    Args:
        article_id (int): 文章 Id
        page (int, optional): 页码. Defaults to 1.
        count (int, optional): 每次获取的评论数（不包含子评论）. Defaults to 10.
        author_only (bool, optional): 为 True 时只获取作者发布的评论，包含作者发布的子评论及其父评论. Defaults to False.
        sorting_method (str, optional): 排序方式，为”positive“时按时间正序排列，为”reverse“时按时间倒序排列. Defaults to "positive".

    Returns:
        list: 评论信息
    """
    order_by = {
        "positive": "asc", 
        "reverse": "desc"
    }[sorting_method]
    json_obj = GetArticleCommentsJsonDataApi(article_id, page, count, author_only, order_by)
    result = []
    for item in json_obj["comments"]:
        item_data = {
            "cmid": item["id"], 
            "publish_time": datetime.fromisoformat(item["created_at"]), 
            "content": item["compiled_content"], 
            "floor": item["floor"], 
            "images": [image["url"] for image in item["images"]], 
            "likes_count": item["likes_count"], 
            "sub_comments_count": item["children_count"], 
            "user": {
                "uid": item["user"]["id"], 
                "name": item["user"]["nickname"], 
                "uslug": item["user"]["slug"], 
                "avatar_url": item["user"]["avatar"]
            }
        }
        try:
            item["member"]
        except KeyError:  # 没有开通会员
            pass
        else:
            item_data["user"]["vip_type"] = {
                    "bronze": "铜牌", 
                    "silver": "银牌", 
                    "gold": "黄金", 
                    "platina": "白金"
            }[item["user"]["member"]["type"]]
            item_data["user"]["vip_expire_date"] = datetime.fromtimestamp(item["user"]["member"]["expires_at"])
        
        try:
            item["children"]
        except KeyError:  # 没有子评论
            pass
        else:
            item_data["sub_comments"] = []
            for sub_comment in item["children"]:
                sub_comment_data = {
                "cmid": sub_comment["id"], 
                "publish_time": datetime.fromisoformat(sub_comment["created_at"]), 
                "content": sub_comment["compiled_content"], 
                "images": [image["url"] for image in sub_comment["images"]], 
                "parent_comment_id": sub_comment["parent_id"], 
                "user": {
                    "uid": sub_comment["user"]["id"], 
                    "name": sub_comment["user"]["nickname"], 
                    "uslug": sub_comment["user"]["slug"], 
                    "avatar_url": sub_comment["user"]["avatar"]
                }
            }
                
                try:
                    sub_comment["user"]["member"]
                except KeyError:  # 没有开通会员
                    pass
                else:
                    sub_comment_data["user"]["vip_type"] = {
                            "bronze": "铜牌",
                            "silver": "银牌" , 
                            "gold": "黄金", 
                            "platina": "白金"
                    }[item["user"]["member"]["type"]]
                    sub_comment_data["user"]["vip_expire_date"] = datetime.fromtimestamp(sub_comment["user"]["member"]["expires_at"])

                item_data["sub_comments"].append(sub_comment_data)
                    
        result.append(item_data)
    return result


from datetime import datetime

import requests
from lxml import etree

from .headers import *

try:
    import simplejson as json
except ImportError:
    import json


def GetArticleJsonDataApi(article_url: str) -> object:
    request_url = article_url.replace("https://www.jianshu.com/", "https://www.jianshu.com/asimov/")
    source = requests.get(request_url, headers=jianshu_request_header).content
    json_obj = json.loads(source)
    return json_obj

def GetArticleHtmlJsonDataApi(article_url: str) -> object:
    source = requests.get(article_url, headers=PC_header).content
    html_obj = etree.HTML(source)
    json_obj = json.loads(html_obj.xpath("//script[@id='__NEXT_DATA__']/text()")[0])
    return json_obj

def GetArticleCommentsJsonDataApi(article_id: int, page: int, count: int, 
                                  author_only: bool, order_by: str) -> object:
    params = {
        "page": page, 
        "count": count, 
        "author_only": author_only, 
        "order_by": order_by
    }
    request_url = "https://www.jianshu.com/shakespeare/notes/" + str(article_id) + "/comments"
    source = requests.get(request_url, params=params, headers=jianshu_request_header).content
    json_obj = json.loads(source)
    return json_obj

def GetBeikeIslandTradeRankListJsonDataApi(ranktype: int, pageIndex: int) -> object:
    params = {
        "ranktype": ranktype, 
        "pageIndex": pageIndex
    }
    source = requests.post("https://www.beikeisland.com/api/Trade/getTradeRankList", 
                            headers=BeikeIsland_request_header, json=params).content
    json_obj = json.loads(source)
    return json_obj

def GetBeikeIslandTradeListJsonDataApi(pageIndex: int, retype: int):
    params = {
        "pageIndex": pageIndex, 
        "retype": retype
    }
    source = requests.post("https://www.beikeisland.com/api/Trade/getTradeList", 
                            headers=BeikeIsland_request_header, json=params).content
    json_obj = json.loads(source)
    return json_obj

def GetCollectionJsonDataApi(collection_url: str) -> object:
    request_url = collection_url.replace("https://www.jianshu.com/c/", "https://www.jianshu.com/asimov/collections/slug/")
    source = requests.get(request_url, headers=jianshu_request_header).content
    json_obj = json.loads(source)
    return json_obj

def GetCollectionEditorsJsonDataApi(collection_id: int, page: int) -> object:
    request_url = "https://www.jianshu.com/collections/" + str(collection_id) + "/editors"
    params = {
        "page": page
    }
    source = requests.get(request_url, params=params, headers=jianshu_request_header).content
    json_obj = json.loads(source)
    return json_obj

def GetCollectionRecommendedWritersJsonDataApi(collection_id: int, page: int, count: int) -> object:
    params = {
        "collection_id": collection_id, 
        "page": page, 
        "count": count
    }
    source = requests.get("https://www.jianshu.com/collections/recommended_users", 
                            params=params, headers=jianshu_request_header).content
    json_obj = json.loads(source)
    return json_obj

def GetCollectionSubscribersJsonDataApi(collection_id: int, max_sort_id: int) -> object:
    request_url = "https://www.jianshu.com/collection/" + str(collection_id) + "/subscribers"
    params = {
        "max_sort_id": max_sort_id
    }
    source = requests.get(request_url, params=params, headers=jianshu_request_header).content
    json_obj = json.loads(source)
    return json_obj

def GetCollectionArticlesJsonDataApi(collection_slug: str, page: int, count: int, order_by: str) -> object:
    request_url = "https://www.jianshu.com/asimov/collections/slug/" + collection_slug + "/public_notes"
    params = {
        "page": page, 
        "count": count, 
        "order_by": order_by
    }
    source = requests.get(request_url, params=params, headers=jianshu_request_header).content
    json_obj = json.loads(source)
    return json_obj

def GetIslandJsonDataApi(island_url: str) -> object:
    request_url = island_url.replace("https://www.jianshu.com/g/", "https://www.jianshu.com/asimov/groups/")
    source = requests.get(request_url, headers=jianshu_request_header).content
    json_obj = json.loads(source)
    return json_obj

def GetIslandPostsJsonDataApi(group_slug: str, max_id: int, 
                           count: int, topic_id: int, order_by: str):
    params = {
        "group_slug": group_slug, 
        "order_by": order_by, 
        "max_id": max_id, 
        "count": count, 
        "topic_id": topic_id
    }
    source = requests.get("https://www.jianshu.com/asimov/posts", 
                            params=params, headers=jianshu_request_header).content
    json_obj = json.loads(source)
    return json_obj

def GetNotebookJsonDataApi(notebook_url: str) -> object:
    request_url = notebook_url.replace("https://www.jianshu.com/", "https://www.jianshu.com/asimov/")
    source = requests.get(request_url, headers=jianshu_request_header).content
    json_obj = json.loads(source)
    return json_obj

def GetNotebookArticlesJsonDataApi(notebook_url: str, page: int, 
                                count: int, order_by: str) -> object:
    request_url = notebook_url.replace("https://www.jianshu.com/nb/", 
            "https://www.jianshu.com/asimov/notebooks/") + "/public_notes/"
    params = {
        "page": page, 
        "count": count, 
        "order_by": order_by
    }
    source = requests.get(request_url, params=params, headers=jianshu_request_header).content
    json_obj = json.loads(source)
    return json_obj

def GetAssetsRankJsonDataApi(max_id: int, since_id: int) -> object:
    params = {
        "max_id": max_id, 
        "since_id": since_id
    }
    source = requests.get("https://www.jianshu.com/asimov/fp_rankings", params=params, headers=jianshu_request_header).content
    json_obj = json.loads(source)
    return json_obj

def GetDailyArticleRankListJsonDataApi() -> object:
    source = requests.get("https://www.jianshu.com/asimov/daily_activity_participants/rank", headers=jianshu_request_header).content
    json_obj = json.loads(source)
    return json_obj

def GetArticlesFPRankListJsonDataApi(date: str, type_: str) -> object:  # 避免覆盖内置函数
    params = {
        "date": date, 
        "type": type_
    }
    source = requests.get("https://www.jianshu.com/asimov/fp_rankings/voter_notes", params=params, headers=jianshu_request_header).content
    json_obj = json.loads(source)
    return json_obj

def GetUserJsonDataApi(user_url: str) -> object:
    request_url = user_url.replace("https://www.jianshu.com/u/", "https://www.jianshu.com/asimov/users/slug/")
    source = requests.get(request_url, headers=jianshu_request_header).content
    json_obj = json.loads(source)
    return json_obj

def GetUserPCHtmlDataApi(user_url: str) -> object:
    source = requests.get(user_url, headers=PC_header).content
    html_obj = etree.HTML(source)
    return html_obj

def GetUserCollectionsAndNotebooksJsonDataApi(user_url: str, user_slug: str) -> object:
    request_url = user_url.replace("/u/", "/users/") + "/collections_and_notebooks"
    params = {
        "slug": user_slug
    }
    source = requests.get(request_url, headers=jianshu_request_header, params=params).content
    json_obj = json.loads(source)
    return json_obj

def GetUserArticlesListJsonDataApi(user_url: str, page: int, 
                                count: int, order_by: str) -> object :
    request_url = user_url.replace("/u/", "/asimov/users/slug/") + "/public_notes"
    params = {
        "page": page, 
        "count": count, 
        "order_by": order_by
    }
    source = requests.get(request_url, headers=jianshu_request_header, params=params).content
    json_obj = json.loads(source)
    return json_obj

def GetUserFollowingListHtmlDataApi(user_url: str, page: int):
    request_url = user_url.replace("/u/", "/users/") + "/following"
    params = {
        "page": page
    }
    source = requests.get(request_url, headers=PC_header, params=params).content
    html_obj = etree.HTML(source)
    return html_obj

def GetUserFollowersListHtmlDataApi(user_url: str, page: int):
    request_url = user_url.replace("/u/", "/users/") + "/followers"
    params = {
        "page": page
    }
    source = requests.get(request_url, headers=PC_header, params=params).content
    html_obj = etree.HTML(source)
    return html_obj



from datetime import datetime

from .basic_apis import (GetBeikeIslandTradeListJsonDataApi,
                         GetBeikeIslandTradeRankListJsonDataApi)
from .convert import UserUrlToUserSlug
from .exceptions import ResourceError


def GetBeikeIslandTotalTradeAmount() -> int:
    """获取贝壳小岛总交易量

    Returns:
        int: 总交易量
    """
    # 不传送数据也能正常获取，节省时间和带宽
    json_obj = GetBeikeIslandTradeRankListJsonDataApi(ranktype=None, pageIndex=None)
    result = json_obj["data"]["totalcount"]
    return result

def GetBeikeIslandTotalTradeCount() -> int:
    """获取贝壳小岛总交易笔数

    Returns:
        int: 总交易笔数
    """
    # 不传送数据也能正常获取，节省时间和带宽
    json_obj = GetBeikeIslandTradeRankListJsonDataApi(ranktype=None, pageIndex=None)
    result = json_obj["data"]["totaltime"]
    return result

def GetBeikeIslandTotalTradeRankData(page: int = 1) -> list:
    """获取贝壳小岛总交易排行榜中的用户数据

    Args:
        page (int, optional): 页码. Defaults to 1.

    Returns:
        list: 总交易排行榜的用户数据
    """
    json_obj = GetBeikeIslandTradeRankListJsonDataApi(ranktype=3, pageIndex=page)
    result = []
    for item in json_obj["data"]["ranklist"]:
        item_data = {
            "bkuid": item["userid"], 
            "jianshuname": item["jianshuname"], 
            "avatar_url": item["avatarurl"], 
            "userurl": item["jianshupath"], 
            "uslug": UserUrlToUserSlug(item["jianshupath"]), 
            "total_trade_amount": item["totalamount"], 
            "total_trade_times": item["totaltime"]
        }
        result.append(item_data)
    return result

def GetBeikeIslandBuyTradeRankData(page: int = 1) -> list:
    """获取贝壳小岛买贝排行榜中的用户数据

    Args:
        page (int, optional): 页码. Defaults to 1.

    Returns:
        list: 买贝榜的用户数据
    """
    json_obj = GetBeikeIslandTradeRankListJsonDataApi(ranktype=1, pageIndex=page)
    result = []
    for item in json_obj["data"]["ranklist"]:
        item_data = {
            "bkuid": item["userid"], 
            "jianshuname": item["jianshuname"], 
            "avatar_url": item["avatarurl"], 
            "userurl": item["jianshupath"], 
            "uslug": UserUrlToUserSlug(item["jianshupath"]), 
            "total_trade_amount": item["totalamount"], 
            "total_trade_times": item["totaltime"]
        }
        result.append(item_data)
    return result

def GetBeikeIslandSellTradeRankData(page: int = 1) -> list:
    """获取贝壳小岛卖贝排行榜中的用户数据

    Args:
        page (int, optional): 页码. Defaults to 1.

    Returns:
        list: 卖贝榜的用户数据
    """
    json_obj = GetBeikeIslandTradeRankListJsonDataApi(ranktype=2, pageIndex=page)
    result = []
    for item in json_obj["data"]["ranklist"]:
        item_data = {
            "bkuid": item["userid"], 
            "jianshuname": item["jianshuname"], 
            "avatar_url": item["avatarurl"], 
            "userurl": item["jianshupath"], 
            "uslug": UserUrlToUserSlug(item["jianshupath"]), 
            "total_trade_amount": item["totalamount"], 
            "total_trade_times": item["totaltime"]
        }
        result.append(item_data)
    return result

def GetBeikeIslandTradeOrderInfo(trade_type: str, page: int = 1) -> list:
    """获取贝壳小岛的挂单信息

    Args:
        trade_type (str): buy 为买单，sell 为卖单
        page (int, optional): 页码. Defaults to 1.

    Returns:
        list: 挂单数据
    """
    # 通过 trade_type 构建 retype
    retype = {
        "buy": 2, 
        "sell": 1
    }[trade_type]
    json_obj = GetBeikeIslandTradeListJsonDataApi(pageIndex=page, retype=retype)
    result = []
    for item in json_obj["data"]["tradelist"]:
        item_data = {
            "tradeid": item["id"], 
            "tradeslug": item["tradeno"],   # ? 我也不确定这个 no 什么意思,回来去问问
            "user":{
            "jianshuname": item["jianshuname"], 
            "bkname": item["reusername"],   # ? 还有个 nickname，不知道哪个对
            "avatar_url": item["avatarurl"], 
            "userlevelcode": item["levelnum"], 
            "userlevel": item["userlevel"], 
            "user_trade_count": item["tradecount"]
            }, 
            "total": item["recount"], 
            "traded": item["recount"] - item["cantradenum"], 
            "remaining": item["cantradenum"], 
            "price": item["reprice"], 
            "minimum_limit": item["minlimit"], 
            "percentage": item["compeletper"], 
            "statuscode": item["statuscode"], 
            "status": item["statustext"], 
            "publish_time": datetime.fromisoformat(item["releasetime"])
        }
        result.append(item_data)
    return result

def GetBeikeIslandTradePrice(trade_type: str, rank: int = 1) -> float:
    """获取特定位置交易单的价格

    Args:
        trade_type (str): buy 为买单，sell 为卖单
        rank (int, optional): 自最低 / 最高价开始，需要获取的价格所在的位置. Defaults to 1.

    Returns:
        float: 交易单的价格
    """
    pageIndex = int(rank / 10)  # 确定需要请求的页码
    # 通过 trade_type 构建 retype
    retype = {
        "buy": 2, 
        "sell": 1
    }[trade_type]
    json_obj = GetBeikeIslandTradeListJsonDataApi(pageIndex=pageIndex, retype=retype)
    rank_in_this_page = rank % 10 - 1  # 本页信息中目标交易单的位置，考虑索引下标起始值为 0 问题
    try:
        result = json_obj["data"]["tradelist"][rank_in_this_page]["reprice"]
    except IndexError:
        raise ResourceError("没有该排名的价格")
    return result



from datetime import datetime

from .assert_funcs import AssertNotebookUrl
from .basic_apis import GetNotebookArticlesJsonDataApi, GetNotebookJsonDataApi


def GetNotebookName(notebook_url: str) -> str:
    """获取文集名称

    Args:
        notebook_url (str): 文集 Url

    Returns:
        str: 文集名称
    """
    AssertNotebookUrl(notebook_url)
    json_obj = GetNotebookJsonDataApi(notebook_url)
    result = json_obj["name"]
    return result

def GetNotebookArticlesCount(notebook_url: str) -> int:
    """获取文集中的文章数量

    Args:
        notebook_url (str): 文集 Url

    Returns:
        int: 文章数量
    """
    AssertNotebookUrl(notebook_url)
    json_obj = GetNotebookJsonDataApi(notebook_url)
    result = json_obj["notes_count"]
    return result

def GetNotebookAuthorName(notebook_url: str) -> str:
    """获取文集的作者名

    Args:
        notebook_url (str): 文集 Url

    Returns:
        int: 作者名
    """
    AssertNotebookUrl(notebook_url)
    json_obj = GetNotebookJsonDataApi(notebook_url)
    result = json_obj["user"]["nickname"]
    return result

def GetNotebookAuthorInfo(notebook_url: str) -> dict:
    """获取文集作者的信息

    Args:
        notebook_url (str): 文集 Url

    Returns:
        list: 作者信息
    """
    AssertNotebookUrl(notebook_url)
    json_obj = GetNotebookJsonDataApi(notebook_url)
    result = {
        "name": json_obj["user"]["nickname"], 
        "uslug": json_obj["user"]["slug"], 
        "avatar_url": json_obj["user"]["avatar"]
    }
    return result

def GetNotebookWordage(notebook_url: str) -> int:
    """获取文集中所有文章的总字数

    Args:
        notebook_url (str): 文集 Url

    Returns:
        int: 文章总字数
    """
    AssertNotebookUrl(notebook_url)
    json_obj = GetNotebookJsonDataApi(notebook_url)
    result = json_obj["wordage"]
    return result

def GetNotebookSubscribersCount(notebook_url: str) -> int:
    """获取文集的关注着数量

    Args:
        notebook_url (str): 文集 Url

    Returns:
        int: 关注着数量
    """
    AssertNotebookUrl(notebook_url)
    json_obj = GetNotebookJsonDataApi(notebook_url)
    result = json_obj["subscribers_count"]
    return result

def GetNotebookUpdateTime(notebook_url: str) -> datetime:
    """获取文集的更新时间

    Args:
        notebook_url (str): 文集 Url

    Returns:
        datetime: 更新时间
    """
    AssertNotebookUrl(notebook_url)
    json_obj = GetNotebookJsonDataApi(notebook_url)
    result = datetime.fromtimestamp(json_obj["last_updated_at"])
    return result

def GetNotebookArticlesInfo(notebook_url: str, page: int = 1, 
                            count: int = 10, sorting_method: str ="time") -> list:
    """获取文集中的文章信息

    Args:
        notebook_url (str): 文集 Url
        page (int, optional): 页码. Defaults to 1.
        count (int, optional): 每次返回的数据数量. Defaults to 10.
        sorting_method (str, optional): 排序方法，time 为按照发布时间排序，
        comment_time 为按照最近评论时间排序，hot 为按照热度排序. Defaults to "time".

    Returns:
        list: 文章信息
    """
    AssertNotebookUrl(notebook_url)
    order_by = {
        "time": "added_at", 
        "comment_time": "commented_at", 
        "hot": "top"
    }[sorting_method]
    json_obj = GetNotebookArticlesJsonDataApi(notebook_url=notebook_url, 
                                           page=page, count=count, order_by=order_by)
    result = []
    for item in json_obj:
        item_data  = {
            "aid": item["object"]["data"]["id"], 
            "title": item["object"]["data"]["title"], 
            "aslug": item["object"]["data"]["slug"], 
            "release_time": datetime.fromisoformat(item["object"]["data"]["first_shared_at"]), 
            "first_image_url": item["object"]["data"]["list_image_url"], 
            "summary": item["object"]["data"]["public_abbr"], 
            "views_count": item["object"]["data"]["views_count"], 
            "likes_count": item["object"]["data"]["likes_count"], 
            "is_top": item["object"]["data"]["is_top"], 
            "paid": item["object"]["data"]["paid"], 
            "commentable": item["object"]["data"]["commentable"], 
            "user": {
                "uid": item["object"]["data"]["user"]["id"], 
                "name": item["object"]["data"]["user"]["nickname"], 
                "uslug": item["object"]["data"]["user"]["slug"], 
                "avatar_url": item["object"]["data"]["user"]["avatar"]
            }, 
            "total_fp_amount": item["object"]["data"]["total_fp_amount"] / 1000, 
            "comments_count": item["object"]["data"]["public_comments_count"], 
            "rewards_count": item["object"]["data"]["total_rewards_count"]
        }
        result.append(item_data)
    return result



from .assert_funcs import (AssertArticleStatusNormal, AssertArticleUrl,
                           AssertCollectionUrl, AssertIslandUrl,
                           AssertNotebookUrl, AssertString, AssertUserUrl)
from .basic_apis import GetArticleJsonDataApi, GetUserJsonDataApi


def UserUrlToUserId(user_url: str) -> int:
    """该函数接收用户个人主页 Url，并将其转换成用户 Id

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        int: 用户 Id
    """
    AssertString(user_url)
    AssertUserUrl(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["id"]
    return result

def UserSlugToUserId(user_slug: str) -> int:
    """该函数接收用户 Slug，并将其转换成用户 Id

    Args:
        user_url (str): 用户 Slug

    Returns:
        int: 用户 Id
    """
    AssertString(user_slug)
    user_url = UserSlugToUserUrl(user_slug)
    AssertUserUrl(user_url)
    result = UserUrlToUserId(user_url)
    return result
    

def UserUrlToUserSlug(user_url: str) -> str:
    """该函数接收用户个人主页 Url，并将其转换成用户 Slug

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        str: 用户 Slug
    """
    AssertString(user_url)
    AssertUserUrl(user_url)
    return user_url.replace("https://www.jianshu.com/u/", "").replace("/", "")

def UserSlugToUserUrl(user_slug: str) -> str:
    """该函数接收用户 Slug，并将其转换成用户个人主页 Url

    Args:
        user_slug (str): 用户 Slug

    Returns:
        str: 用户个人主页 Url
    """
    AssertString(user_slug)
    result = "https://www.jianshu.com/u/" + user_slug
    AssertUserUrl(result)
    return result


def ArticleUrlToArticleSlug(article_url: str) -> str:
    """该函数接收文章 Url，并将其转换成文章 Slug

    Args:
        article_url (str): 文章 Url

    Returns:
        str: 文章 Slug
    """
    AssertString(article_url)
    AssertArticleUrl(article_url)
    return article_url.replace("https://www.jianshu.com/p/", "")

def ArticleSlugToArticleUrl(article_slug: str) -> str:
    """该函数接收文章 Slug，并将其转换成文章 Url

    Args:
        article_slug (str): 文章 Slug

    Returns:
        str: 文章 Url
    """
    AssertString(article_slug)
    result = "https://www.jianshu.com/p/" + article_slug
    AssertArticleUrl(result)
    return result

def ArticleSlugToArticleId(article_url: str) -> int:
    """该函数接收文章 Slug，并将其转换成文章 Id

    Args:
        article_slug (str): 文章 Slug

    Returns:
        int: 文章 Id
    """
    AssertString(article_url)
    AssertArticleUrl(article_url)
    AssertArticleStatusNormal(article_url)
    json_obj = GetArticleJsonDataApi(article_url)
    result = json_obj["id"]
    return result


def NotebookUrlToNotebookId(notebook_url: str) -> int:
    """该函数接收文集 Url，并将其转换成文集 Id

    Args:
        notebook_url (str): 文集 Url

    Returns:
        int: 文集 Id
    """
    AssertString(notebook_url)
    AssertNotebookUrl(notebook_url)
    json_obj = GetArticleJsonDataApi(notebook_url)
    result = json_obj["id"]
    return result

def NotebookUrlToNotebookSlug(notebook_url: str) -> str:
    """该函数接收文集 Url，并将其转换成文集 Slug

    Args:
        notebook_url (str): 文集 Url

    Returns:
        str: 文集 Slug
    """
    AssertString(notebook_url)
    AssertNotebookUrl(notebook_url)
    return notebook_url.replace("https://www.jianshu.com/nb/", "")

def NotebookSlugToNotebookUrl(notebook_slug: str) -> str:
    """该函数接收文集 Slug，并将其转换成文集 Url

    Args:
        notebook_slug (str): 文集 Slug

    Returns:
        str: 文集 Url
    """
    AssertString(notebook_slug)
    result = "https://www.jianshu.com/nb/" + notebook_slug
    AssertNotebookUrl(result)
    return result


def CollectionUrlToCollectionSlug(collection_url: str) -> str:
    """该函数接收专题 Url，并将其转换成专题 Slug

    Args:
        collection_url (str): 专题 Url

    Returns:
        str: 专题 Slug
    """
    AssertString(collection_url)
    AssertCollectionUrl(collection_url)
    return collection_url.replace("https://www.jianshu.com/c/", "")

def CollectionSlugToCollectionUrl(collection_slug: str) -> str:
    """该函数接收专题 Slug，并将其转换成专题 Url

    Args:
        collection_slug (str): 专题 Slug

    Returns:
        str: 专题 Url
    """
    AssertString(collection_slug)
    result = "https://www.jianshu.com/c/" + collection_slug
    AssertCollectionUrl(result)
    return result


def IslandUrlToIslandSlug(island_url: str) -> str:
    """该函数接收小岛 Url，并将其转换成小岛 Slug

    Args:
        island_url (str): 小岛 Url

    Returns:
        str: 小岛 Slug
    """
    AssertString(island_url)
    AssertIslandUrl(island_url)
    return island_url.replace("https://www.jianshu.com/g/", "")

def IslandSlugToIslandUrl(island_slug: str) -> str:
    """该函数接收小岛 Slug，并将其转换成小岛 Url

    Args:
        island_slug (str): 小岛 Slug

    Returns:
        str: 小岛 Url
    """
    AssertString(island_slug)
    result = "https://www.jianshu.com/g/" + island_slug
    AssertIslandUrl(result)
    return result

def UserUrlToUserUrlScheme(user_url: str) -> str:
    """该函数接收用户个人主页 Url，并返回跳转到简书 App 中对应用户的 Url Scheme

    Args:
        user_url (str): 用户个人主页 Url
    Returns:
        str: 跳转到简书 App 中对应用户的 Url Scheme
    """
    AssertString(user_url)
    AssertUserUrl(user_url)
    result = user_url.replace("https://www.jianshu.com/u/", "jianshu://u/")
    return result

def ArticleUrlToArticleUrlScheme(article_url: str) -> str:
    """该函数接收文章 Url，并返回跳转到简书 App 中对应文章的 Url Scheme

    Args:
        article_url (str): 文章 Url
    Returns:
        str: 跳转到简书 App 中对应文章的 Url Scheme
    """
    AssertString(article_url)
    AssertArticleUrl(article_url)
    result = article_url.replace("https://www.jianshu.com/p/", "jianshu://notes/")
    return result

def NotebookUrlToNotebookUrlScheme(notebook_url: str) -> str:
    """该函数接收文集 Url，并返回跳转到简书 App 中对应文集的 Url Scheme

    Args:
        notebook_url (str): 文集 Url
    Returns:
        str: 跳转到简书 App 中对应文集的 Url Scheme
    """
    AssertString(notebook_url)
    AssertNotebookUrl(notebook_url)
    result = notebook_url.replace("https://www.jianshu.com/nb/", "jianshu://nb/")
    return result

def CollectionUrlToCollectionUrlScheme(collection_url: str) -> str:
    """该函数接收专题 Url，并返回跳转到简书 App 中对应专题的 Url Scheme

    Args:
        collection_url (str): 专题 Url
    Returns:
        str: 跳转到简书 App 中对应专题的 Url Scheme
    """
    AssertString(collection_url)
    AssertCollectionUrl(collection_url)
    result = collection_url.replace("https://www.jianshu.com/c/", "jianshu://c/")
    return result



from datetime import datetime

from .assert_funcs import AssertIslandUrl
from .basic_apis import GetIslandJsonDataApi, GetIslandPostsJsonDataApi
from .convert import IslandUrlToIslandSlug


def GetIslandName(island_url: str) -> str:
    """获取小岛名称

    Args:
        island_url (str): 小岛 Url

    Returns:
        str: 小岛名称
    """
    AssertIslandUrl(island_url)
    json_obj = GetIslandJsonDataApi(island_url)
    result = json_obj["name"]
    return result

def GetIslandAvatarUrl(island_url: str) -> str:
    """获取小岛头像链接

    Args:
        island_url (str): 小岛 Url

    Returns:
        str: 小岛头像链接
    """
    AssertIslandUrl(island_url)
    json_obj = GetIslandJsonDataApi(island_url)
    result = json_obj["image"]
    return result

def GetIslandIntroduction(island_url: str) -> str:
    """获取小岛简介

    Args:
        island_url (str): 小岛 Url

    Returns:
        str: 小岛简介
    """
    AssertIslandUrl(island_url)
    json_obj = GetIslandJsonDataApi(island_url)
    result = json_obj["intro"]
    return result

def GetIslandMembersCount(island_url: str) -> int:
    """获取小岛成员数量

    Args:
        island_url (str): 小岛 Url

    Returns:
        int: 成员数量
    """
    AssertIslandUrl(island_url)
    json_obj = GetIslandJsonDataApi(island_url)
    result = json_obj["members_count"]
    return result

def GetIslandPostsCount(island_url: str) -> int:
    """获取小岛帖子数量

    Args:
        island_url (str): 小岛 Url

    Returns:
        int: 帖子数量
    """
    AssertIslandUrl(island_url)
    json_obj = GetIslandJsonDataApi(island_url)
    result = json_obj["posts_count"]
    return result

def GetIslandCategory(island_url: str) -> str:
    """获取小岛分类

    Args:
        island_url (str): 小岛 Url

    Returns:
        str: 分类
    """
    AssertIslandUrl(island_url)
    json_obj = GetIslandJsonDataApi(island_url)
    result = json_obj["category"]["name"]
    return result

def GetIslandPosts(island_url: str, start_sort_id: int = None, count: int = 10, 
                    topic_id: int = None, sorting_method: str ="time") -> list:
    """获取小岛帖子信息

        Args:
            island_url (str): 小岛 Url
            start_sort_id (int, optional): 起始序号，等于上一条数据的序号. Defaults to None.
            count (int, optional): 每次返回的数据数量. Defaults to 10.
            topic_id (int, optional): 话题 Id. Defaults to None.
            sorting_method (str, optional): 排序方法，time 为按照发布时间排序，
        comment_time 为按照最近评论时间排序，hot 为按照热度排序. Defaults to "time".

        Returns:
            list: 帖子信息
    """
    AssertIslandUrl(island_url)
    order_by = {
        "time": "latest", 
        "hot": "hot", 
        "most_valuable": "best"
    }[sorting_method], 
    json_obj = GetIslandPostsJsonDataApi(group_slug=IslandUrlToIslandSlug(island_url), 
                                      max_id=start_sort_id, count=count, topic_id=topic_id, order_by=order_by)

    result = []
    for item in json_obj:
        item_data = {
            "sorted_id": item["sorted_id"], 
            "pid": item["id"], 
            "pslug": item["slug"], 
            "title": item["title"], 
            "content": item["content"],   # TODO: 这里获取到的信息不完整
            # "images": item["images"]
            "likes_count": item["likes_count"], 
            "comments_count": item["comments_count"], 
            "release_time": datetime.fromtimestamp(item["created_at"]), 
            "is_hot": item["is_hot"], 
            "is_most_valuable": item["is_best"], 
            "is_topped": item["is_top"], 
            "is_new": item["is_new"], 
            "island": {
                "iid": item["group"]["id"], 
                "islug": item["group"]["slug"], 
                "island_name": item["group"]["name"]
            }, 
            "user": {
                "uid": item["user"]["id"], 
                "uslug": item["user"]["slug"], 
                "user_name": item["user"]["nickname"], 
                "avatar_url": item["user"]["avatar"]
                # "badge": item["user"]["badge"]["text"]
                # 有个 member 不知道干什么用的，没解析
            }
            # "topic": {
            #     "tid": item["topic"]["id"], 
            #     "tslug": item["topic"]["slug"], 
            #     "topic_name": item["topic"]["name"]
            #     # 有个 group_role 不知道干什么用的，没解析
            # }
        }
        try:
            image_urls = []
            for image in item["images"]:
                image_urls.append(image["url"])
        except KeyError:
            pass  # 没有图片则跳过
        try: 
            item_data["user"]["badge"] = item["user"]["badge"]["text"]
        except KeyError:
            pass  # 没有徽章则跳过
        try:
            item_data["topic"] = {
                "tid": item["topic"]["id"], 
                "tslug": item["topic"]["slug"], 
                "topic_name": item["topic"]["name"]
                # 有个 group_role 不知道干什么用的，没解析
            }
        except KeyError:
            pass  # 没有话题则跳过
        result.append(item_data)
    return result



class InputError(Exception):
    """该错误意味着传入的参数有误，如不是对应类型的 Url
    """
    pass

class APIError(Exception):
    """该错误意味着在函数内部的方法调用中出现了问题，一般与传入的参数无关
    """
    pass

class ResourceError(Exception):
    """该错误意味着访问的资源出现问题，如不存在或者审核中
    """


from datetime import datetime
from re import findall, sub

from lxml import etree

from .assert_funcs import AssertUserUrl, AssertUserStatusNormal
from .basic_apis import (GetUserArticlesListJsonDataApi,
                         GetUserCollectionsAndNotebooksJsonDataApi,
                         GetUserFollowersListHtmlDataApi,
                         GetUserFollowingListHtmlDataApi, GetUserJsonDataApi,
                         GetUserPCHtmlDataApi)
from .convert import UserUrlToUserSlug
from .exceptions import APIError


def GetUserName(user_url: str) -> str:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的昵称

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        str: 用户昵称
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["nickname"]
    return result

def GetUserGender(user_url: str) -> int:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的性别

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        int: 用户性别，0 为未知，1 为男，2 为女
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["gender"]
    return result

def GetUserFollowersCount(user_url: str) -> int:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的关注数

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        int: 用户关注数
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["following_users_count"]
    return result

def GetUserFansCount(user_url: str) -> int:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的粉丝数

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        int: 用户粉丝数
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["followers_count"]
    return result

def GetUserArticlesCount(user_url: str) -> int:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的文章数

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        int: 用户文章数
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    html_obj = GetUserPCHtmlDataApi(user_url)
    result = html_obj.xpath("//div[@class='info']/ul/li[3]/div[@class='meta-block']/a/p")[0].text
    result = int(result)
    return result

def GetUserWordage(user_url: str) -> int:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的文章总字数

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        int: 用户文章总字数
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["total_wordage"]
    return result

def GetUserLikesCount(user_url: str) -> int:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的被喜欢数

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        int: 用户被喜欢数
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["total_likes_count"]
    return result

def GetUserAssetsCount(user_url: str) -> float:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的总资产
    
    # ! 当用户资产大于 10000 时，结果的精确度将下降到 1000

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        float: 用户总资产
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    html_obj = GetUserPCHtmlDataApi(user_url)
    try:
        result = html_obj.xpath("//div[@class='info']/ul/li[6]/div[@class='meta-block']/p")[0].text
    except IndexError:
        raise APIError("受简书网页展示限制，无法获取该用户的总资产")
    result = float(result.replace(".", "").replace("w", "000"))
    return result

def GetUserFPCount(user_url: str) -> float:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的简书钻数量

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        float: 用户简书钻数量
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["jsd_balance"] / 1000
    if json_obj["total_wordage"] == 0 and result == 0:
        raise APIError("受简书限制，无法获取该用户的简书钻数量")
    return result

def GetUserFTNCount(user_url: str) -> float:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的简书贝数量

    # ! 视用户资产配置情况不同，该函数获取到的数值会有不大于 1000 的偏差

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        float: 用户简书贝数量
    """
    assets = GetUserAssetsCount(user_url)
    FTN = GetUserFPCount(user_url)
    result = assets - FTN
    result = abs(result)  # 处理用户简书贝数量较少导致结果为负的情况
    result = round(result, 3)  # 处理浮点数精度问题
    return result

def GetUserBadgesList(user_url: str) -> list:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的徽章列表

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        list: 用户徽章列表
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    html_obj = GetUserPCHtmlDataApi(user_url)
    result = html_obj.xpath("//li[@class='badge-icon']/a/text()")
    result = [item.replace(" ", "").replace("\n", "") for item in result]  # 移除空格和换行符
    result = [item for item in result if item != ""]  # 去除空值
    return result

def GetUserLastUpdateTime(user_url: str) -> datetime:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的文章最近更新时间

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        datetime: 用户文章最近更新时间
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = datetime.fromtimestamp(json_obj["last_updated_at"])
    return result

def GetUserVIPInfo(user_url: str) -> dict:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的会员信息

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        dict: 用户会员信息
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    try:
        result = {
            "vip_type": {
                "bronze": "铜牌",
                "silver": "银牌" , 
                "gold": "黄金", 
                "platina": "白金"
            }[json_obj["member"]["type"]], 
            "expire_date": datetime.fromtimestamp(json_obj["member"]["expires_at"])
        }
    except KeyError:
        result = {
            "vip_type": None, 
            "expire_date": None
        }
    return result

def GetUserIntroductionHtml(user_url: str) -> str:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的个人简介

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        str: Html 格式的用户个人简介
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    result = json_obj["intro"]
    return result

def GetUserIntroductionText(user_url: str) -> str:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的个人简介

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        str: 纯文本格式的用户个人简介
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    json_obj = GetUserJsonDataApi(user_url)
    if json_obj["intro"] == "":   # 简介为空
        return ""
    html_obj = etree.HTML(json_obj["intro"])
    result = html_obj.xpath("//*/text()")
    result = "\n".join(result)
    return result

def GetUserNotebooksInfo(user_url: str) -> list:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的文集与连载信息

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        list: 用户文集与连载信息
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    json_obj = GetUserCollectionsAndNotebooksJsonDataApi(user_url=user_url, user_slug=UserUrlToUserSlug(user_url))
    result = []
    for item in json_obj["notebooks"]:
        item_data = {
            "nid": item["id"], 
            "name": item["name"], 
            "is_book": item["book"]
        }
        if item["book"] == True:
            item_data["is_paid_book"] = item["paid_book"]  # 如果是连载，则判断是否是付费连载
        result.append(item_data)
    return result

def GetUserOwnCollectionsInfo(user_url: str) -> list:
    """该函数接收用户个人主页 Url，并返回该链接对应用户自己创建的专题信息

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        list: 用户自己创建的专题信息
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    json_obj = GetUserCollectionsAndNotebooksJsonDataApi(user_url=user_url, user_slug=UserUrlToUserSlug(user_url))
    result = []
    for item in json_obj["own_collections"]:
        item_data = {
            "cid": item["id"], 
            "cslug": item["slug"], 
            "name": item["title"], 
            "avatar_url": item["avatar"]
        }
        result.append(item_data)
    return result

def GetUserManageableCollectionsInfo(user_url: str) -> list:
    """该函数接收用户个人主页 Url，并返回该链接对应用户有管理权限的专题信息

    Args:
        user_url (str): 用户个人主页 Url

    Returns:
        list: 用户有管理权限的专题信息
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    json_obj = GetUserCollectionsAndNotebooksJsonDataApi(user_url=user_url, user_slug=UserUrlToUserSlug(user_url))
    result = []
    for item in json_obj["manageable_collections"]:
        item_data = {
            "cid": item["id"], 
            "cslug": item["slug"], 
            "name": item["title"], 
            "avatar_url": item["avatar"]
        }
        result.append(item_data)
    return result

def GetUserArticlesInfo(user_url: str, page: int = 1, count: int = 10, 
                        sorting_method: str ="time") -> list:
    """该函数接收用户个人主页 Url，并返回该链接对应用户的文章信息

    Args:
        user_url (str): 用户个人主页 Url
        page (int, optional): 页码，与网页端文章顺序相同. Defaults to 1.
        count (int, optional): 获取的文章数量. Defaults to 10.
        sorting_method (str, optional): 排序方法，time 为按照发布时间排序，
        comment_time 为按照最近评论时间排序，hot 为按照热度排序. Defaults to "time".

    Returns:
        list: 用户文章信息
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    order_by = {
        "time": "added_at", 
        "comment_time": "commented_at", 
        "hot": "top"
    }[sorting_method]
    json_obj = GetUserArticlesListJsonDataApi(user_url=user_url, page=page, 
                                           count=count, order_by=order_by)
    result = []
    for item in json_obj:
        item_data  = {
            "aid": item["object"]["data"]["id"], 
            "title": item["object"]["data"]["title"], 
            "aslug": item["object"]["data"]["slug"], 
            "release_time": datetime.fromisoformat(item["object"]["data"]["first_shared_at"]), 
            "first_image_url": item["object"]["data"]["list_image_url"], 
            "summary": item["object"]["data"]["public_abbr"], 
            "views_count": item["object"]["data"]["views_count"], 
            "likes_count": item["object"]["data"]["likes_count"], 
            "is_top": item["object"]["data"]["is_top"], 
            "paid": item["object"]["data"]["paid"], 
            "commentable": item["object"]["data"]["commentable"], 
            "user": {
                "uid": item["object"]["data"]["user"]["id"], 
                "name": item["object"]["data"]["user"]["nickname"], 
                "uslug": item["object"]["data"]["user"]["slug"], 
                "avatar_url": item["object"]["data"]["user"]["avatar"]
            }, 
            "total_fp_amount": item["object"]["data"]["total_fp_amount"] / 1000, 
            "comments_count": item["object"]["data"]["public_comments_count"], 
            "rewards_count": item["object"]["data"]["total_rewards_count"]
        }
        result.append(item_data)
    return result

def GetUserFollowingInfo(user_url: str, page:int = 1) -> list:
    """该函数接收用户个人主页 Url 和页码，并返回该用户关注列表中对应页数的用户信息

    Args:
        user_url (str): 用户个人主页 Url
        page (int, optional): 关注列表页码. Defaults to 1.

    Returns:
        list: 该用户关注列表中对应页数的用户信息
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    html_obj = GetUserFollowingListHtmlDataApi(user_url=user_url, page=page)
    name_raw_data = html_obj.xpath("//a[@class='name']")[1:]
    if name_raw_data == []:  # 判断该页数据是否为空
        return []
    followers_raw_data = html_obj.xpath("//div[@class='meta'][1]/span[1]")
    fans_raw_data = html_obj.xpath("//div[@class='meta'][1]/span[2]")
    articles_raw_data = html_obj.xpath("//div[@class='meta'][1]/span[3]")
    words_and_likes_raw_data = html_obj.xpath("//div[@class='meta'][2]")
    result = []
    for index in range(8):
        item_data = {
            "name": name_raw_data[index].text, 
            "followers_count": int(followers_raw_data[index].text.replace("关注 ", "")), 
            "fans_count": int(fans_raw_data[index].text.replace("粉丝", "")), 
            "articles_count": int(articles_raw_data[index].text.replace("文章 ", "")), 
            "words_count": int(findall(r"\d+", words_and_likes_raw_data[index].text)[0]),   # TODO: 重复运行正则匹配，影响效率，需要优化
            "likes_count": int(findall(r"\d+", words_and_likes_raw_data[index].text)[1])
        }
        result.append(item_data)
    return result

def GetUserFansInfo(user_url: str, page:int = 1) -> list:
    """该函数接收用户个人主页 Url 和页码，并返回该用户粉丝列表中对应页数的用户信息

    Args:
        user_url (str): 用户个人主页 Url
        page (int, optional): 粉丝列表页码. Defaults to 1.

    Returns:
        list: 该用户粉丝列表中对应页数的用户信息
    """
    AssertUserUrl(user_url)
    AssertUserStatusNormal(user_url)
    html_obj = GetUserFollowersListHtmlDataApi(user_url=user_url, page=page)
    name_raw_data = html_obj.xpath("//a[@class='name']")[1:]
    if name_raw_data == []:  # 判断该页数据是否为空
        return []
    followers_raw_data = html_obj.xpath("//div[@class='meta'][1]/span[1]")
    fans_raw_data = html_obj.xpath("//div[@class='meta'][1]/span[2]")
    articles_raw_data = html_obj.xpath("//div[@class='meta'][1]/span[3]")
    words_and_likes_raw_data = html_obj.xpath("//div[@class='meta'][2]")
    result = []
    for index in range(8):
        item_data = {
            "name": name_raw_data[index].text, 
            "followers_count": int(followers_raw_data[index].text.replace("关注 ", "")), 
            "fans_count": int(fans_raw_data[index].text.replace("粉丝", "")), 
            "articles_count": int(articles_raw_data[index].text.replace("文章 ", "")), 
            "words_count": int(findall(r"\d+", words_and_likes_raw_data[index].text)[0]),   # TODO: 重复运行正则匹配，影响效率，需要优化
            "likes_count": int(findall(r"\d+", words_and_likes_raw_data[index].text)[1])
        }
        result.append(item_data)
    return result



from datetime import datetime
from hashlib import md5

from . import article, beikeisland, collection, island, notebook, user
from .assert_funcs import (AssertArticleUrl, AssertCollectionUrl,
                           AssertIslandUrl, AssertNotebookUrl, AssertUserUrl)
from .convert import (ArticleSlugToArticleUrl, ArticleUrlToArticleSlug,
                      CollectionSlugToCollectionUrl,
                      CollectionUrlToCollectionSlug, IslandSlugToIslandUrl,
                      IslandUrlToIslandSlug, NotebookSlugToNotebookUrl,
                      NotebookUrlToNotebookId, NotebookUrlToNotebookSlug,
                      UserSlugToUserUrl, UserUrlToUserSlug)
from .exceptions import InputError


def GetHash(*args: any) -> str:
    """获取任意数量参数的哈希值

    Returns:
        str: 哈希值前 6 位
    """
    data = "".join([str(item) for item in args])
    result = md5(data.encode("utf-8")).hexdigest()
    result = result[0:7]  # 取前 6 位
    return result

def SimpleCache(cache_obj: list, getting_func: object, args: dict, disable_cache: bool = False) -> any:
    """基本缓存

    Args:
        cache_obj (list): 缓存信息储存对象
        getting_func (object): 信息获取函数
        args (dict): 信息获取函数的参数
        disable_cache (bool, optional): 禁用缓存. Defaults to False.

    Returns:
        any: 信息获取函数的返回值
    """
    if cache_obj != [] and disable_cache == False:
        return cache_obj[0]
    else:
        result = getting_func(**args)
        cache_obj.append(result)
        return result

def HashCache(cache_obj: dict, getting_func: object, args: dict, disable_cache: bool = False) -> any:
    """基于哈希值的缓存

    Args:
        cache_obj (dict): 缓存信息储存对象
        getting_func (object): 信息获取函数
        args (dict): 信息获取函数的参数
        disable_cache (bool, optional): 禁用缓存. Defaults to False.

    Returns:
        any: 信息获取函数的返回值
    """
    hash = GetHash(*list(args.values()))
    cache = cache_obj.get(hash)
    if cache != None and disable_cache == False:
        return cache
    else:
        result = getting_func(**args)
        cache_obj[hash] = result
        return result

class User():
    """用户类
    """
    def __init__(self, source: str):
        """构建新的用户对象

        Args:
            source (str): 用户个人主页 Url 或用户 Slug
        """
        try:
            AssertUserUrl(source)
        except InputError:
            # TODO: 这里还有点问题
            source = UserSlugToUserUrl(source)
            AssertUserUrl(source)
        self._url = source

        self._slug = []
        self._name = []
        self._gender = []
        self._followers_count = []
        self._fans_count = []
        self._articles_count = []
        self._wordage = []
        self._likes_count = []
        self._assets_count = []
        self._FP_count = []
        self._FTN_count = []
        self._badges = []
        self._last_update_time = []
        self._VIP_info = []
        self._introduction_text = []
        self._introduction_html = []
        self._notebooks = []
        self._own_collections = []
        self._manageable_collections = []

        self._articles_info = {}
        self._followers_info = {}
        self._fans_info = {}
    
    @property
    def url(self) -> str:
        """获取用户主页 Url

        Returns:
            str: 用户主页 Url
        """
        return self._url
    
    @property
    def slug(self, disable_cache: bool = False) -> str:
        """获取用户 Slug

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 用户 Slug
        """
        result = SimpleCache(self._slug, UserUrlToUserSlug, 
                            {"user_url": self._url})
        return result
    
    @property
    def name(self, disable_cache:bool = False) -> str:
        """获取用户昵称

        Args:
            disable_cache (bool, optional): 禁用缓存，强制请求新结果. Defaults to False.

        Returns:
            str: 用户昵称
        """
        result = SimpleCache(self._name, user.GetUserName, 
                            {"user_url": self._url}, disable_cache)
        return result

    @property
    def gender(self, disable_cache: bool = False) -> int:
        """获取用户性别

        Args:
            disable_cache (bool, optional): 禁用缓存，强制请求新结果. Defaults to False.

        Returns:
            int: 用户性别，0 为未知，1 为男，2 为女
        """
        result = SimpleCache(self._gender, user.GetUserGender, 
                             {"user_url": self._url}, disable_cache)
        return result
    
    @property
    def followers_count(self, disable_cache: bool = False) -> int:
        """获取用户关注数

        Args:
            disable_cache (bool, optional): 禁用缓存，强制请求结果. Defaults to False.

        Returns:
            int: 关注数
        """
        result = SimpleCache(self._followers_count, user.GetUserFollowersCount, 
                            {"user_url": self._url}, disable_cache)
        return result
    
    @property
    def fans_count(self, disable_cache: bool = False) -> int:
        """获取用户粉丝数

        Args:
            disable_cache (bool, optional): 禁用缓存，强制请求结果. Defaults to False.

        Returns:
            int: 粉丝数
        """
        result = SimpleCache(self._fans_count, user.GetUserFansCount, 
                            {"user_url": self._url}, disable_cache)
        return result
    
    @property
    def articles_count(self, disable_cache: bool = False) -> int:
        """获取用户文章数

        Args:
            disable_cache (bool, optional): 禁用缓存，强制请求结果. Defaults to False.

        Returns:
            int: 文章数
        """
        result = SimpleCache(self._articles_count, user.GetUserArticlesCount, 
                            {"user_url": self._url}, disable_cache)
        return result
    
    @property
    def wordage(self, disable_cache: bool = False) -> int:
        """获取用户总字数

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 总字数
        """
        result = SimpleCache(self._wordage, user.GetUserWordage, 
                            {"user_url": self._url}, disable_cache)
        return result
    
    @property
    def likes_count(self, disable_cache: bool = False) -> int:
        """获取用户被点赞数

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 被点赞数
        """
        result = SimpleCache(self._likes_count, user.GetUserLikesCount, 
                            {"user_url": self._url}, disable_cache)
        return result

    @property
    def assets_count(self, disable_cache:bool = False) -> int:
        """获取用户资产量

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 资产量
        """
        result = SimpleCache(self._assets_count, user.GetUserAssetsCount, 
                            {"user_url": self._url}, disable_cache)
        return result
    
    @property
    def FP_count(self, disable_cache: bool = False) -> int:
        """获取用户简书钻数量

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 简书钻数量
        """
        result = SimpleCache(self._FP_count, user.GetUserFPCount, 
                            {"user_url": self._url}, disable_cache)
        return result
    
    @property
    def FTN_count(self, disable_cache: bool = False) -> int:
        """获取用户简书贝数量

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 简书贝数量
        """
        result = SimpleCache(self._FTN_count, user.GetUserFTNCount, 
                            {"user_url": self._url}, disable_cache)
        return result
    
    @property
    def badges(self, disable_cache: bool = False) -> list:
        """获取徽章列表

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            list: 徽章列表
        """
        result = SimpleCache(self._badges, user.GetUserBadgesList, 
                            {"user_url": self._url}, disable_cache)
        return result
    
    @property
    def last_update_time(self, disable_cache: bool = False) -> datetime:
        """获取最近更新时间

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            datetime: 最近更新时间
        """
        result = SimpleCache(self._last_update_time, user.GetUserLastUpdateTime, 
                             {"user_url": self._url}, disable_cache)
        return result
    
    @property
    def VIP_info(self, disable_cache: bool = False) -> dict:
        """获取用户会员信息

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            dict: 会员信息
        """
        result = SimpleCache(self._VIP_info, user.GetUserVIPInfo, 
                             {"user_url": self._url}, disable_cache)
        return result
    
    @property
    def introduction_text(self, disable_cache: bool = False) -> str:
        """获取纯文本格式的用户简介

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str:纯文本格式的用户简介
        """
        result = SimpleCache(self._introduction_text, user.GetUserIntroductionText, 
                             {"user_url", self._url}, disable_cache)
        return result
    
    @property
    def introduction_html(self, disable_cache: bool = False) -> str:
        """获取 Html 格式的用户简介

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: Html 格式的用户简介
        """
        result = SimpleCache(self._introduction_html, user.GetUserIntroductionHtml, 
                            {"user_url": self._url}, disable_cache)
        return result
    
    @property
    def notebooks(self, disable_cache: bool = False) -> list:
        """获取用户文集信息

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            list: 文集信息
        """
        result = SimpleCache(self._notebooks, user.GetUserNotebooksInfo, 
                            {"user_url": self._url}, disable_cache)
        return result
    
    @property
    def own_collections(self, disable_cache: bool = False) -> list:
        """获取自己创建的专题信息

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            list: 自己创建的专题信息
        """
        result = SimpleCache(self._own_collections, user.GetUserOwnCollectionsInfo, 
                            {"user_url": self._url}, disable_cache)
        return result
    
    @property
    def manageable_collections(self, disable_cache: bool = False) -> list:
        """获取有管理权的专题信息

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            list: 有管理权的专题信息
        """
        result = SimpleCache(self._manageable_collections, user.GetUserManageableCollectionsInfo, 
                            {"user_url": self._url}, disable_cache)
        return result
    
    def articles_info(self, page: int = 1, count: int = 10, 
                        disable_cache: bool = False) -> list:
        """获取文章信息

        Args:
            page (int, optional): 页码. Defaults to 1.
            count (int, optional): 每次获取的文章信息数量. Defaults to 10.
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            list: 文章信息
        """
        result = HashCache(self._articles_info, user.GetUserArticlesInfo, 
                            {"user_url": self._url, "page": page, "count": count}, 
                            disable_cache)
        return result

    def followers_info(self, page: int = 1, disable_cache: bool = False) -> list:
        """获取关注者信息

        Args:
            page (int, optional): 页码. Defaults to 1.
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            list: 关注者信息
        """
        result = HashCache(self._followers_info, user.GetUserFollowersInfo, 
                            {"user_url": self._url, "page": page}, disable_cache)
        return result

    def fans_info(self, page: int = 1, disable_cache: bool = False) -> list:
        """获取粉丝信息

        Args:
            page (int, optional): 页码. Defaults to 1.
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            list: 粉丝信息
        """
        result = HashCache(self._fans_info, user.GetUserFansInfo, 
                            {"user_url": self._url, "page": page}, disable_cache)
        return result
    
    def __eq__(self, other: object) -> bool:
        """判断是否是同一个用户

        Args:
            other (object): 另一个对象

        Returns:
            bool: 判断结果
        """
        if isinstance(other, User) == False:
            return False  # 不是由用户类构建的必定不相等
        if self._slug == other._slug:
            return True
        else:
            return False

    def __str__(self) -> str:
        """输出用户信息摘要

        Returns:
            str: 用户信息摘要
        """
        result = "用户信息摘要：\n用户名：{}\n性别：{}\n关注数：{}\n粉丝数：{}\n文章数：{}\n总字数：{}\n被点赞数：{}\n总资产：{}\n简书钻：{}\n简书贝：{}\n会员等级：{}\n会员过期时间：{}".format(
            self.name, {0: "未知", 1: "男", 2: "女"}[self.gender], self.followers_count, 
            self.fans_count, self.articles_count, self.wordage, self.likes_count, self.assets_count, 
            self.FP_count, self.FTN_count, self.VIP_info["vip_type"], self.VIP_info["expire_date"]
        )
        return result

class Article():
    """文章类
    """
    def __init__(self, source: str):
        """构建新的文章对象

        Args:
            source (str): 文章 Url 或文章 Slug
        """
        try:
            AssertArticleUrl(source)
        except InputError:
            source = ArticleSlugToArticleUrl(source)
            AssertArticleUrl(source)
        self._url = source

        self._slug = []
        self._title = []
        self._author = []
        self._wordage = []
        self._reads_count = []
        self._likes_count = []
        self._comments_count = []
        self._most_valuable_comments_count = []
        self._total_FP_count = []
        self._description = []
        self._publish_time = []
        self._update_time = []
        self._paid_status = []
        self._reprint_status = []
        self._comment_status = []
        self._html = []
        self._text = []
    
    @property
    def url(self) -> str:
        """获取文章 Url

        Returns:
            str: 文章 Url
        """
        return self._url
    
    @property
    def slug(self, disable_cache: bool = False) -> str:
        """获取文章 Slug

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 文章 Slug
        """
        result = SimpleCache(self._slug, ArticleUrlToArticleSlug, 
                            {"user_url": self._slug}, disable_cache)
        return result
    
    @property
    def title(self, disable_cache: bool = False) -> str:
        """获取文章标题

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 标题
        """
        result = SimpleCache(self._title, article.GetArticleTitle, 
                            {"article_url": self._url}, disable_cache)
        return result
    
    @property
    def author_name(self, disable_cache: bool = False) -> str:
        """获取文章作者名

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 作者名
        """
        result = SimpleCache(self._author, article.GetArticleAuthorName, 
                            {"article_url": self._url}, disable_cache)
        return result
    
    @property
    def wordage(self, disable_cache: bool = False) -> int:
        """获取文章总字数

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 总字数
        """
        result = SimpleCache(self._wordage, article.GetArticleWordage, 
                            {"article_url": self._url}, disable_cache)
        return result
    
    @property
    def reads_count(self, disable_cache: bool = False) -> int:
        """获取文章阅读量

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 阅读量
        """
        result = SimpleCache(self._reads_count, article.GetArticleReadsCount, 
                            {"article_url": self._url}, disable_cache)
        return result
    
    @property
    def likes_count(self, disable_cache: bool = False) -> int:
        """获取文章点赞量

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 文章点赞量
        """
        result = SimpleCache(self._likes_count, article.GetArticleLikesCount, 
                            {"article_url": self._url}, disable_cache)
        return result
    
    @property
    def comments_count(self, disable_cache: bool = False) -> int:
        """获取文章评论量

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 文章评论量
        """
        result = SimpleCache(self._comments_count, article.GetArticleCommentsCount, 
                            {"article_url": self._url}, disable_cache)
        return result
    
    @property
    def most_valuable_comments_count(self, disable_cache: bool = False) -> int:
        """获取文章精选评论量

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 文章精选评论量
        """
        result = SimpleCache(self._most_valuable_comments_count, 
                            article.GetArticleMostValuableCommentsCount, 
                            {"article_url": self._url}, disable_cache)
        return result
    
    @property
    def total_FP_count(self, disable_cache: bool = False) -> int:
        """获取文章总获钻量

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 文章总获钻量
        """
        result = SimpleCache(self._total_FP_count, article.GetArticleTotalFPCount, 
                            {"article_url": self._url}, disable_cache)
        return result

    @property
    def description(self, disable_cache: bool = False) -> str:
        """获取文章摘要

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 文章摘要
        """
        result = SimpleCache(self._description, article.GetArticleDescription, 
                            {"article_url": self._url}, disable_cache)
        return result
    
    @property
    def publish_time(self, disable_cache: bool = False) -> datetime:
        """获取文章发布时间

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            datetime: 文章发布时间
        """
        result = SimpleCache(self._publish_time, article.GetArticlePublishTime, 
                            {"article_url": self._url}, disable_cache)
        return result
    
    @property
    def update_time(self, disable_cache: bool = False) -> datetime:
        """获取文章更新时间

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            datetime: 文章更新时间
        """
        result = SimpleCache(self._update_time, article.GetArticleUpdateTime, 
                            {"article_url": self._url}, disable_cache)
        return result

    @property
    def paid_status(self, disable_cache: bool = False) -> bool:
        """获取文章付费状态

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            bool: 文章付费状态
        """
        result = SimpleCache(self._paid_status, article.GetArticlePaidStatus, 
                            {"article_url": self._url}, disable_cache)
        return result

    @property
    def reprint_status(self, disable_cache: bool = False) -> bool:
        """获取文章转载状态

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            bool: 文章转载状态
        """
        result = SimpleCache(self._reprint_status, article.GetArticleReprintStatus, 
                            {"article_url": self._url}, disable_cache)
        return result
    
    @property
    def comment_status(self, disable_cache: bool = False) -> bool:
        """获取文章评论状态

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            bool: 文章评论状态
        """
        result = SimpleCache(self._comment_status, article.GetArticleCommentStatus, 
                            {"article_url": self._url}, disable_cache)
        return result
    
    @property
    def html(self, disable_cache: bool = False) -> str:
        """获取 Html 格式的文章内容

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: Html 格式的文章内容
        """
        result = SimpleCache(self._html, article.GetArticleHtml, 
                            {"article_url": self._url}, disable_cache)
        return result
    
    @property
    def text(self, disable_cache: bool = False) -> str:
        """获取纯文本格式的文章内容

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 纯文本格式的文章内容
        """
        result = SimpleCache(self._text, article.GetArticleText, 
                            {"article_url": self._url}, disable_cache)
        return result
    
    def __eq__(self, other: object) -> bool:
        """判断是否是同一篇文章

        Args:
            other (object): 另一个对象

        Returns:
            bool: 判断结果
        """
        if isinstance(other, Article) == False:
            return False  # 不是由文章类构建的必定不相等
        if self._slug == other._slug:
            return True
        else:
            return False
    
    def __str__(self) -> str:
        result = "文章信息摘要：\n标题：{}\n作者：{}\n获钻量：{}\n发布时间：{}\n更新时间：{}\n字数：{}\n阅读量：{}\n点赞量：{}\n评论量：{}".format(
            self.title, self.author_name, self.total_FP_count, self.publish_time, \
            self.update_time, self.wordage, self.reads_count, self.likes_count, self.comments_count
        )
        return result

class Notebook():
    """文集类
    """
    def __init__(self, source: str):
        """构建新的文集对象

        Args:
            source (str): 文集 Url 或文集 Slug
        """
        try:
            AssertNotebookUrl(source)
        except InputError:
            source = NotebookSlugToNotebookUrl(source)
            AssertNotebookUrl(source)
        self._url = source

        self._id = []
        self._slug = []
        self._name = []
        self._articles_count = []
        self._author_name = []
        self._author_info = []
        self._wordage = []
        self._subscribers_count = []
        self._update_time = []
        self._articles_info = {}
        
    @property
    def url(self) -> str:
        """获取文集 Url

        Returns:
            str: 文集 Url
        """
        return self._url
    
    @property
    def id(self, disable_cache: bool = False) -> int:
        """获取文集 Id

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 文集 Id
        """
        result = SimpleCache(self._id, NotebookUrlToNotebookId, 
                             {"notebook_url": self._url}, disable_cache)
        return result
    
    @property
    def slug(self, disable_cache: bool = False) -> str:
        """获取文集 Slug

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 文集 Slug
        """
        result = SimpleCache(self._slug, NotebookUrlToNotebookSlug, 
                             {"notebook_url": self._url}, disable_cache)
        return result

    @property
    def name(self, disable_cache: bool = False) -> str:
        """获取文集名称

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 文集名称
        """
        result = SimpleCache(self._name, notebook.GetNotebookName, 
                             {"notebook_url": self._url}, disable_cache)
        return result
    
    @property
    def articles_count(self, disable_cache: bool = False) -> int:
        """获取文集中的文章总数

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 文章总数
        """
        result = SimpleCache(self._articles_count, notebook.GetNotebookArticlesCount, 
                             {"notebook_url": self._url}, disable_cache)
        return result

    @property
    def author_name(self, disable_cache: bool = False) -> str:
        """获取文集的作者名

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 作者名
        """
        result = SimpleCache(self._author_name, notebook.GetNotebookAuthorName, 
                             {"notebook_url": self._url}, disable_cache)
        return result
    
    @property
    def author_info(self, disable_cache: bool = False) -> dict:
        """获取文集的作者信息

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            dict: 作者信息
        """
        result = SimpleCache(self._author_info, notebook.GetNotebookAuthorInfo, 
                             {"notebook_url": self._url}, disable_cache)
        return result
    
    @property
    def wordage(self, disable_cache: bool = False) -> int:
        """获取文集中所有文章的总字数

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 文集总字数
        """
        result = SimpleCache(self._wordage, notebook.GetNotebookWordage, 
                             {"notebook_url": self._url}, disable_cache)
        return result
    
    @property
    def subscribers_count(self, disable_cache: bool = False) -> int:
        """获取文集的关注者数量

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 关注者数量
        """
        result = SimpleCache(self._subscribers_count, notebook.GetNotebookSubscribersCount, 
                             {"notebook_url": self._url}, disable_cache)
        return result
    
    @property
    def update_time(self, disable_cache: bool = False) -> datetime:
        """获取文集的更新时间

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            datetime: 更新时间
        """
        result = SimpleCache(self._update_time, notebook.GetNotebookUpdateTime, 
                             {"notebook_url": self._url}, disable_cache)
        return result
    
    def articles_info(self, page: int = 1, count: int = 10, sorting_method: str ="time", disable_cache: bool = False) -> list:
        """获取文集中的文章信息

        Args:
            page (int, optional): 页码. Defaults to 1.
            count (int, optional): 每次返回的数据数量. Defaults to 10.
            sorting_method (str, optional): 排序方法，time 为按照发布时间排序，
        comment_time 为按照最近评论时间排序，hot 为按照热度排序. Defaults to "time".
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            list: 文章信息
        """
        result = HashCache(self._articles_info, notebook.GetNotebookArticlesInfo, 
                           {"notebook_url": self._url, "page": page, "count": count, 
                            "sorting_method": sorting_method}, disable_cache)
        return result

    def __eq__(self, other: object) -> bool:
        """判断是否是同一个文集

        Args:
            other (object): 另一个对象

        Returns:
            bool: 判断结果
        """
        if isinstance(other, Notebook) == False:
            return False  # 不是由文集类构建的必定不相等
        if self.slug == other.slug:
            return True
        else:
            return False
        
    def __str__(self) -> str:
        """输出文集信息摘要

        Returns:
            str: 文集信息摘要
        """
        result = "文具信息摘要：\n名称：{}\n作者：{}\n文章数：{}\n总字数：{}\n关注者数量：{}\n更新时间：{}".format(
            self.name, self.author_name, self.articles_count, self.wordage, 
            self.subscribers_count, self.update_time
        )
        return result
    
class Collection():
    """专题类
    """
    def __init__(self, source: str, id: int = None):
        """构建新的文集对象

        Args:
            source (str): 文集 Url 或文集 Slug
            id (int): 专题 Id，如不传入将无法获取编辑、推荐作者和关注者信息
        """
        try:
            AssertCollectionUrl(source)
        except InputError:
            source = CollectionSlugToCollectionUrl(source)
            AssertCollectionUrl(source)
        self._url = source
        self._id = id
        
        self._slug = []
        self._name = []
        self._avatar_url = []
        self._introduction_text = []
        self._introduction_html = []
        self._articles_update_time = []
        self._info_update_time = []
        self._articles_count = []
        self._subscribers_count = []
        self._owner_info = []
        
        self._editors_info = {}
        self._recommended_writers_info = {}
        self._subscribers_info = {}
        self._articles_info = {}
        
    @property
    def url(self) -> str:
        """获取专题 Url

        Returns:
            str: 专题 Url
        """
        return self._url
    
    @property
    def slug(self, disable_cache: bool = False) -> str:
        """获取专题 Slug

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 专题 Slug
        """
        result = SimpleCache(self._slug, CollectionUrlToCollectionSlug, 
                            {"user_url": self._slug}, disable_cache)
        return result
    
    @property
    def name(self, disable_cache: bool = False) -> str:
        """获取专题名称

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 专题名称
        """
        result = SimpleCache(self._name, collection.GetCollectionName, 
                             {"collection_url": self._url}, disable_cache)
        return result
    
    @property
    def avatar_url(self, disable_cache: bool = False) -> str:
        """获取专题头像

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 专题头像
        """
        result = SimpleCache(self._name, collection.GetCollectionAvatarUrl, 
                             {"collection_url": self._url}, disable_cache)
        return result
    
    @property
    def introduction_text(self, disable_cache: bool = False) -> str:
        """获取纯文本格式的专题简介

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 纯文本格式的专题简介
        """
        result = SimpleCache(self._introduction_text, collection.GetCollectionIntroductionText, 
                             {"collection_url": self._url}, disable_cache)
        return result
    
    @property
    def introduction_html(self, disable_cache: bool = False) -> str:
        """获取 Html 格式的专题简介

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str:  Html 格式的专题简介
        """
        result = SimpleCache(self._introduction_html, collection.GetCollectionIntroductionHtml, 
                             {"collection_url": self._url}, disable_cache)
        return result
    
    @property
    def articles_update_time(self, disable_cache: bool = False) -> datetime:
        """获取专题文章更新时间

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            datetime: 专题文章更新时间
        """
        result = SimpleCache(self._articles_update_time, collection.GetCollectionArticlesUpdateTime, 
                             {"collection_url": self._url}, disable_cache)
        return result

    @property
    def info_update_time(self, disable_cache: bool = False) -> datetime:
        """获取专题信息更新时间

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            datetime: 专题信息更新时间
        """
        result = SimpleCache(self._info_update_time, collection.GetCollectionInfoUpdateTime, 
                             {"collection_url": self._url}, disable_cache)
        return result
    
    @property
    def owner_info(self, disable_cache: bool = False) -> dict:
        """获取专题的所有者信息

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            dict: 用户信息
        """
        result = SimpleCache(self._owner_info, collection.GetCollectionOwnerInfo, 
                             {"collection_url": self._url}, disable_cache)
        return result
    
    @property
    def articles_count(self, disable_cache: bool = False) -> int:
        """获取专题文章数

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 专题文章数
        """
        result = SimpleCache(self._articles_count, collection.GetCollectionArticlesCount, 
                             {"collection_url": self._url}, disable_cache)
        return result
    
    @property
    def subscribers_count(self, disable_cache: bool = False) -> int:
        """获取专题关注者数

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 专题关注者数
        """
        result = SimpleCache(self._subscribers_count, collection.GetCollectionSubscribersCount, 
                             {"collection_url": self._url}, disable_cache)
        return result
    
    def editors_info(self, page: int = 1, disable_cache: bool = False) -> list:
        """获取专题编辑信息

        Args:
            page (int, optional): 页码. Defaults to 1.
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Raises:
            InputError: 因缺少 Id 参数而无法获取结果时抛出此异常

        Returns:
            list: 编辑信息
        """
        if self._id == None:
            raise InputError("实例化该专题对象时未传入 Id 参数，无法获取编辑信息")
        result = HashCache(self._editors_info, collection.GetCollectionEditorsInfo, 
                           {"collection_id": self._id, "page": page}, disable_cache)
        return result

    def recommended_writers_info(self, page: int = False, disable_cache: bool = False) -> list:
        """获取专题推荐作者信息

        Args:
            page (int, optional): 页码. Defaults to False.
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Raises:
            InputError: 因缺少 Id 参数而无法获取结果时抛出此异常

        Returns:
            list: 推荐作者信息
        """
        if self._id == None:
            raise InputError("实例化该专题对象时未传入 Id 参数，无法获取推荐作者信息")
        result = HashCache(self._recommended_writers_info, collection.GetCollectionRecommendedWritersInfo, 
                           {"collection_id": self._id}, disable_cache)
        return result
    
    def subscribers_info(self, start_sort_id: int, disable_cache: bool = False) -> list:
        """获取专题关注者信息

        Args:
            start_sort_id (int): 起始序号，等于上一条数据的序号
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Raises:
            InputError: 因缺少 Id 参数而无法获取结果时抛出此异常

        Returns:
            list: 关注者信息
        """
        if self._id == None:
            raise InputError("实例化该专题对象时未传入 Id 参数，无法获取关注着信息")
        result = HashCache(self._subscribers_info, collection.GetCollectionSubscribersInfo, 
                           {"collection_id": self._id, "start_sort_id": start_sort_id}, disable_cache)
        return result
    
    def articles_info(self, page: int = 1, count: int = 10, 
                      sorting_method: str ="time", disable_cache: bool = False) -> list:
        """获取专题文章信息

        Args:
            page (int, optional): 页码. Defaults to 1.
            count (int, optional): 每次返回的数据数量. Defaults to 10.
            sorting_method (str, optional): 排序方法，time 为按照发布时间排序，
        comment_time 为按照最近评论时间排序，hot 为按照热度排序. Defaults to "time".
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            list: 文章信息
        """
        result = HashCache(self._articles_info, collection.GetCollectionArticlesInfo, 
                           {"collection_url": self._url, "page": page, "count": count, 
                            "sorting_method": sorting_method}, disable_cache)
        return result
    
    def __eq__(self, other: object) -> bool:
        """判断是否是同一个专题

        Args:
            other (object): 另一个对象

        Returns:
            bool: 判断结果
        """
        if isinstance(other, Collection) == False:
            return False  # 不是由专题类构建的必定不相等
        if self._slug == other._slug:
            return True
        else:
            return False
    
    def __str__(self) -> str:
        """输出专题信息摘要

        Returns:
            str: 专题信息摘要
        """
        result = "专题信息摘要：\n名称：{}\n文章更新时间：{}\n信息更新时间：{}\n所有者：{}\n收录文章数：{}\n关注者数：{}".format(
            self.name, self.articles_update_time, self.info_update_time, 
            self.owner_info["name"], self.articles_count, self.subscribers_count
        )
        return result

class Island():
    """小岛类
    """
    def __init__(self, source):
        """构建新的小岛对象

        Args:
            source (str): 小岛 Url 或小岛 Slug
        """
        try:
            AssertIslandUrl(source)
        except InputError:
            source = IslandSlugToIslandUrl(source)
            AssertIslandUrl(source)
        self._url = source
        
        self._slug = []
        self._name = []
        self._avatar_url = []
        self._introduction = []
        self._members_count = []
        self._posts_count = []
        self._category = []
        
        self._posts = {}
    
    @property
    def url(self) -> str:
        """获取小岛 Url

        Returns:
            str: 小岛 Url
        """
        return self._url
    
    @property
    def slug(self, disable_cache: bool = False) -> str:
        """获取小岛 Slug

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 小岛 Slug
        """
        result = SimpleCache(self._slug, IslandUrlToIslandSlug, 
                            {"user_url": self._slug}, disable_cache)
        return result
    
    @property
    def name(self, disable_cache: bool = False) -> str:
        """获取小岛名称

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 小岛名称
        """
        result = SimpleCache(self._name, island.GetIslandName, 
                             {"island_url": self._url}, disable_cache)
        return result
    
    @property
    def avatar_url(self, disable_cache: bool = False) -> str:
        result = SimpleCache(self._avatar_url, island.GetIslandAvatarUrl, 
                             {"collection_url":self._url}, disable_cache)
        return result
    
    @property
    def introduction(self, disable_cache: bool = False) -> str:
        """获取小岛简介

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 小岛简介
        """
        result = SimpleCache(self._introduction, island.GetIslandIntroduction, 
                             {"island_url": self._url}, disable_cache)
        return result
    
    @property
    def members_count(self, disable_cache: bool = False) -> int:
        """获取小岛成员数量

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 成员数量
        """
        result = SimpleCache(self._members_count, island.GetIslandMembersCount, 
                             {"island_url": self._url}, disable_cache)
        return result
    
    @property
    def posts_count(self, disable_cache: bool = False) -> int:
        """获取小岛帖子数量

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            int: 帖子数量
        """
        result = SimpleCache(self._posts_count, island.GetIslandPostsCount, 
                             {"island_url": self._url}, disable_cache)
        return result
    
    @property
    def category(self, disable_cache: bool = False) -> str:
        """获取小岛分类

        Args:
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            str: 分类
        """
        result = SimpleCache(self._category, island.GetIslandCategory, 
                             {"island_url": self._url}, disable_cache)
        return result
    
    def posts(self, start_sort_id: int = None, count: int = 10, 
              topic_id: int = None, sorting_method: str ="time", disable_cache: bool = False) -> list:
        """获取小岛帖子信息

        Args:
            start_sort_id (int, optional): 起始序号，等于上一条数据的序号. Defaults to None.
            count (int, optional): 每次返回的数据数量. Defaults to 10.
            topic_id (int, optional): 话题 Id. Defaults to None.
            sorting_method (str, optional): 排序方法，time 为按照发布时间排序，
        comment_time 为按照最近评论时间排序，hot 为按照热度排序. Defaults to "time".
            disable_cache (bool, optional): 禁用缓存. Defaults to False.

        Returns:
            list: 帖子信息
        """
        result = HashCache(self._posts, island.GetIslandPosts, 
                           {"island_url": self._url, "start_sort_id": start_sort_id, 
                            "count": count, "topic_id": topic_id, "sorting_method": sorting_method}, disable_cache)
        return result
    
    def __eq__(self, other: object) -> bool:
        """判断是否是同一个小岛

        Args:
            other (object): 另一个对象

        Returns:
            bool: 判断结果
        """
        if isinstance(other, Collection) == False:
            return False  # 不是由小岛类构建的必定不相等
        if self._slug == other._slug:
            return True
        else:
            return False
    
    def __str__(self) -> str:
        """输出小岛信息摘要

        Returns:
            str: 小岛信息摘要
        """
        result = "小岛信息摘要：\n名称：{}\n成员数量：{}\n帖子数量：{}\n分类：{}".format(
            self.name, self.members_count, self.posts_count,self.category
        )
        return result



import setuptools
from JianshuResearchTools import __version__

with open("README.md","r",encoding = "utf-8") as file:
    long_description = file.read()

setuptools.setup(
    name="JianshuResearchTools",
    version=__version__,
    author="FHU-yezi",
    author_email="yehaowei20060411@qq.com",
    description="科技赋能创作星辰",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/FHU-yezi/JianshuResearchTools",
    packages=["JianshuResearchTools"],
    install_requires=["lxml>=4.6.3","requests>=2.25.1"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6"
)


from datetime import datetime

from .basic_apis import (GetArticlesFPRankListJsonDataApi,
                         GetAssetsRankJsonDataApi,
                         GetDailyArticleRankListJsonDataApi)
from .convert import UserSlugToUserUrl
from .exceptions import APIError, ResourceError
from .user import GetUserAssetsCount


def GetAssetsRankData(start_id: int = 1, get_full: bool = False) -> list:
    """该函数接收一个起始位置参数和一个获取全部数据的布尔值，并返回自该位置后 20 名用户的资产数据

    Args:
        start_id (int, optional): 起始位置. Defaults to 1.
        get_full (bool, optional): 为 True 时获取简书贝和总资产数据，为 False 时不获取. Defaults to False.

    Returns:
        list: 用户资产数据
    """
    since_id = start_id - 1  # 索引下标为 0
    json_obj = GetAssetsRankJsonDataApi(max_id=1000000000, since_id=since_id)
    result = []
    for item in json_obj["rankings"]:
        item_data = {
            "ranking": item["ranking"], 
            "uid": item["user"]["id"], 
            "uslug": item["user"]["slug"], 
            "name": item["user"]["nickname"], 
            "avatar_url": item["user"]["avatar"], 
            "FP": item["amount"] / 1000
        }
        if get_full == True:
            user_url = UserSlugToUserUrl(item_data["uslug"])
            try:
                item_data["Assets"] = GetUserAssetsCount(user_url)
                item_data["FTN"] =  round(item_data["Assets"] - item_data["FP"], 3) # 处理浮点数精度问题
            except APIError:
                pass
        result.append(item_data)
    return result

def GetDailyArticleRankData() -> list:
    """该函数返回日更排行榜的用户信息

    Returns:
        list: 日更排行榜用户信息
    """
    json_obj = GetDailyArticleRankListJsonDataApi()
    result = []
    for item in json_obj["daps"]:
        item_data = {
            "ranking": item["rank"], 
            "uslug": item["slug"], 
            "name": item["nickname"], 
            "avatar_url": item["avatar"], 
            "check_in_count": item["checkin_count"]
        }
        result.append(item_data)
    return result

def GetArticleFPRankData(date: str ="latest") -> list:
    """该函数接收一个日期，并返回对应日期的文章收益排行榜数据

    目前只能获取 2020 年 6 月 20 日之后的数据。

    Args:
        date (str, optional): 日期，格式“YYYYMMDD”. Defaults to "latest".

    Raises:
        ResourceError: 对应日期的排行榜数据为空时会抛出此异常

    Returns:
        list: 对应日期的文章收益排行榜数据
    """
    if date == "latest":
        date = (datetime.date.today() + datetime.timedelta(days=-1)).strftime("%Y%m%d")
    json_obj = GetArticlesFPRankListJsonDataApi(date=date, type_=None)
    if json_obj["notes"] == []:
        raise ResourceError("对应日期的排行榜数据为空")
    result = []
    for ranking, item in enumerate(json_obj["notes"]):
        item_data = {
            "ranking": ranking, 
            "aslug": item["slug"], 
            "title": item["title"], 
            "author_name": item["author_nickname"], 
            "author_avatar_url": item["author_avatar"],
            "fp_to_author": item["author_fp"] / 1000, 
            "fp_to_voter": item["voter_fp"] / 1000, 
            "total_fp": item["fp"] / 1000
        }
        result.append(item_data)
    return result

def GetArticleFPRankBasicInfo(date: str ="latest") -> dict:
    """获取指定日期的文章收益排行榜基础信息
    
    目前只能获取 2020 年 6 月 20 日之后的数据。

    Args:
        date (str, optional): 日期，格式“YYYYMMDD”. Defaults to "latest".

    Raises:
        ResourceError: 对应日期的排行榜数据为空时会抛出此异常

    Returns:
        dict: 文章收益排行榜基础信息
    """
    if date == "latest":
        date = (datetime.date.today() + datetime.timedelta(days=-1)).strftime("%Y%m%d")
    json_obj = GetArticlesFPRankListJsonDataApi(date=date, type_=None)
    if json_obj["notes"] == []:
        raise ResourceError("对应日期的排行榜数据为空")
    result = {
        "total_fp": json_obj["fp"], 
        "fp_to_author": json_obj["author_fp"], 
        "fp_to_voter": json_obj["voter_fp"]
    }
    return result

def GetUserFPRankData(date: str ="latest", rank_type: str ="all") -> list:
    """该函数接收一个日期，并返回对应日期的用户收益排行榜数据

    目前只能获取 2020 年 6 月 20 日之后的数据。

    Args:
        date (str, optional): 日期，格式“YYYYMMDD”. Defaults to "latest".

    Raises:
        ResourceError: 对应日期的排行榜数据为空时会抛出此异常

    Returns:
        list: 对应日期的用户收益排行榜数据
    """
    type_ = {
        "all": None, 
        "write": "note", 
        "vote": "like"
    }[rank_type]
    json_obj = GetArticlesFPRankListJsonDataApi(date=date, type_=type_)
    if json_obj["users"] == []:
        raise ResourceError("对应日期的排行榜数据为空")
    result = []
    for ranking, item in enumerate(json_obj["users"]):
        item_data = {
            "ranking": ranking, 
            "uslug": item["slug"], 
            "name": item["nickname"], 
            "avatar_url": item["avatar"], 
            "fp_from_write": item["author_fp"], 
            "fp_from_vote": item["voter_fp"]
        }
        result.append(item_data)
    return result
